@@ -6,47 +6,51 @@ import binascii
 import cgi
 import datetime
 import functools
+import inspect
+import netrc
 import os
 import re
 import sys
 import time
-import warnings
 import weakref
 from collections import namedtuple
+from collections.abc import Mapping
+from contextlib import suppress
 from math import ceil
 from pathlib import Path
-from time import gmtime
 from urllib.parse import quote
+from urllib.request import getproxies
 
-from async_timeout import timeout
+import async_timeout
+import attr
+from multidict import MultiDict
+from yarl import URL
 
 from . import hdrs
-from .abc import AbstractCookieJar
+from .abc import AbstractAccessLogger
+from .log import client_logger
 
 
-try:
-    from asyncio import ensure_future
-except ImportError:
-    ensure_future = asyncio.async
+__all__ = ('BasicAuth', 'ChainMapProxy')
 
-PY_34 = sys.version_info < (3, 5)
-PY_35 = sys.version_info >= (3, 5)
-PY_352 = sys.version_info >= (3, 5, 2)
+PY_36 = sys.version_info >= (3, 6)
+PY_37 = sys.version_info >= (3, 7)
 
-if sys.version_info >= (3, 4, 3):
-    from http.cookies import SimpleCookie  # noqa
-else:
-    from .backport_cookies import SimpleCookie  # noqa
-
-
-__all__ = ('BasicAuth', 'create_future', 'parse_mimetype',
-           'Timeout', 'ensure_future', 'noop', 'DummyCookieJar')
+if not PY_37:
+    import idna_ssl
+    idna_ssl.patch_match_hostname()
 
 
 sentinel = object()
-Timeout = timeout
 NO_EXTENSIONS = bool(os.environ.get('AIOHTTP_NO_EXTENSIONS'))
 
+# N.B. sys.flags.dev_mode is available on Python 3.7+, use getattr
+# for compatibility with older versions
+DEBUG = (getattr(sys.flags, 'dev_mode', False) or
+         (not sys.flags.ignore_environment and
+          bool(os.environ.get('PYTHONASYNCIODEBUG'))))
+
+
 CHAR = set(chr(i) for i in range(0, 128))
 CTL = set(chr(i) for i in range(0, 32)) | {chr(127), }
 SEPARATORS = {'(', ')', '<', '>', '@', ',', ';', ':', '\\', '"', '/', '[', ']',
@@ -54,31 +58,10 @@ SEPARATORS = {'(', ')', '<', '>', '@', ',', ';', ':', '\\', '"', '/', '[', ']',
 TOKEN = CHAR ^ CTL ^ SEPARATORS
 
 
-class _CoroGuard:
-    __slots__ = ('_coro', '_msg', '_awaited')
-
-    def __init__(self, coro, msg):
-        self._coro = coro
-        self._msg = msg
-        self._awaited = False
-
-    def __iter__(self):
-        self._awaited = True
-        return self._coro.__iter__()
-
-    if PY_35:
-        def __await__(self):
-            self._awaited = True
-            return (yield from self._coro)
-
-    def __del__(self):
-        self._coro = None
-        if not self._awaited:
-            warnings.warn(self._msg, DeprecationWarning)
-
-
 coroutines = asyncio.coroutines
 old_debug = coroutines._DEBUG
+
+# prevent "coroutine noop was never awaited" warning.
 coroutines._DEBUG = False
 
 
@@ -87,27 +70,11 @@ def noop(*args, **kwargs):
     return
 
 
-def deprecated_noop(message):
-    return _CoroGuard(noop(), message)
-
-
 coroutines._DEBUG = old_debug
 
 
-try:
-    from asyncio import isfuture
-except ImportError:
-    def isfuture(fut):
-        return isinstance(fut, asyncio.Future)
-
-
 class BasicAuth(namedtuple('BasicAuth', ['login', 'password', 'encoding'])):
-    """Http basic authentication helper.
-
-    :param str login: Login
-    :param str password: Password
-    :param str encoding: (optional) encoding ('latin1' by default)
-    """
+    """Http basic authentication helper."""
 
     def __new__(cls, login, password='', encoding='latin1'):
         if login is None:
@@ -124,8 +91,7 @@ class BasicAuth(namedtuple('BasicAuth', ['login', 'password', 'encoding'])):
 
     @classmethod
     def decode(cls, auth_header, encoding='latin1'):
-        """Create a :class:`BasicAuth` object from an ``Authorization`` HTTP
-        header."""
+        """Create a BasicAuth object from an Authorization HTTP header."""
         split = auth_header.strip().split(' ')
         if len(split) == 2:
             if split[0].strip().lower() != 'basic':
@@ -143,50 +109,133 @@ class BasicAuth(namedtuple('BasicAuth', ['login', 'password', 'encoding'])):
 
         return cls(username, password, encoding=encoding)
 
+    @classmethod
+    def from_url(cls, url, *, encoding='latin1'):
+        """Create BasicAuth from url."""
+        if not isinstance(url, URL):
+            raise TypeError("url should be yarl.URL instance")
+        if url.user is None:
+            return None
+        return cls(url.user, url.password or '', encoding=encoding)
+
     def encode(self):
         """Encode credentials."""
         creds = ('%s:%s' % (self.login, self.password)).encode(self.encoding)
         return 'Basic %s' % base64.b64encode(creds).decode(self.encoding)
 
 
-if PY_352:
-    def create_future(loop):
-        return loop.create_future()
-else:
-    def create_future(loop):  # pragma: no cover
-        """Compatibility wrapper for the loop.create_future() call introduced in
-        3.5.2."""
-        return asyncio.Future(loop=loop)
+def strip_auth_from_url(url):
+    auth = BasicAuth.from_url(url)
+    if auth is None:
+        return url, None
+    else:
+        return url.with_user(None), auth
+
+
+def netrc_from_env():
+    netrc_obj = None
+    netrc_path = os.environ.get('NETRC')
+    try:
+        if netrc_path is not None:
+            netrc_path = Path(netrc_path)
+        else:
+            home_dir = Path.home()
+            if os.name == 'nt':  # pragma: no cover
+                netrc_path = home_dir.joinpath('_netrc')
+            else:
+                netrc_path = home_dir.joinpath('.netrc')
+
+        if netrc_path and netrc_path.is_file():
+            try:
+                netrc_obj = netrc.netrc(str(netrc_path))
+            except (netrc.NetrcParseError, OSError) as e:
+                client_logger.warning(".netrc file parses fail: %s", e)
+
+        if netrc_obj is None:
+            client_logger.warning("could't find .netrc file")
+    except RuntimeError as e:  # pragma: no cover
+        """ handle error raised by pathlib """
+        client_logger.warning("could't find .netrc file: %s", e)
+    return netrc_obj
+
+
+@attr.s(frozen=True, slots=True)
+class ProxyInfo:
+    proxy = attr.ib(type=str)
+    proxy_auth = attr.ib(type=BasicAuth)
+
+
+def proxies_from_env():
+    proxy_urls = {k: URL(v) for k, v in getproxies().items()
+                  if k in ('http', 'https')}
+    netrc_obj = netrc_from_env()
+    stripped = {k: strip_auth_from_url(v) for k, v in proxy_urls.items()}
+    ret = {}
+    for proto, val in stripped.items():
+        proxy, auth = val
+        if proxy.scheme == 'https':
+            client_logger.warning(
+                "HTTPS proxies %s are not supported, ignoring", proxy)
+            continue
+        if netrc_obj and auth is None:
+            auth_from_netrc = netrc_obj.authenticators(proxy.host)
+            if auth_from_netrc is not None:
+                # auth_from_netrc is a (`user`, `account`, `password`) tuple,
+                # `user` and `account` both can be username,
+                # if `user` is None, use `account`
+                *logins, password = auth_from_netrc
+                auth = BasicAuth(logins[0] if logins[0] else logins[-1],
+                                 password)
+        ret[proto] = ProxyInfo(proxy, auth)
+    return ret
 
 
 def current_task(loop=None):
     if loop is None:
         loop = asyncio.get_event_loop()
 
-    task = asyncio.Task.current_task(loop=loop)
+    if PY_37:
+        task = asyncio.current_task(loop=loop)
+    else:
+        task = asyncio.Task.current_task(loop=loop)
     if task is None:
+        # this should be removed, tokio must use register_task and family API
         if hasattr(loop, 'current_task'):
             task = loop.current_task()
 
     return task
 
 
+def isasyncgenfunction(obj):
+    if hasattr(inspect, 'isasyncgenfunction'):
+        return inspect.isasyncgenfunction(obj)
+    return False
+
+
+@attr.s(frozen=True, slots=True)
+class MimeType:
+    type = attr.ib(type=str)
+    subtype = attr.ib(type=str)
+    suffix = attr.ib(type=str)
+    parameters = attr.ib(type=MultiDict)
+
+
 def parse_mimetype(mimetype):
     """Parses a MIME type into its components.
 
-    :param str mimetype: MIME type
+    mimetype is a MIME type string.
 
-    :returns: 4 element tuple for MIME type, subtype, suffix and parameters
-    :rtype: tuple
+    Returns a MimeType object.
 
     Example:
 
     >>> parse_mimetype('text/html; charset=utf-8')
-    ('text', 'html', '', {'charset': 'utf-8'})
+    MimeType(type='text', subtype='html', suffix='',
+             parameters={'charset': 'utf-8'})
 
     """
     if not mimetype:
-        return '', '', '', {}
+        return MimeType(type='', subtype='', suffix='', parameters={})
 
     parts = mimetype.split(';')
     params = []
@@ -195,7 +244,7 @@ def parse_mimetype(mimetype):
             continue
         key, value = item.split('=', 1) if '=' in item else (item, '')
         params.append((key.lower().strip(), value.strip(' "')))
-    params = dict(params)
+    params = MultiDict(params)
 
     fulltype = parts[0].strip().lower()
     if fulltype == '*':
@@ -205,12 +254,13 @@ def parse_mimetype(mimetype):
         if '/' in fulltype else (fulltype, '')
     stype, suffix = stype.split('+', 1) if '+' in stype else (stype, '')
 
-    return mtype, stype, suffix, params
+    return MimeType(type=mtype, subtype=stype, suffix=suffix,
+                    parameters=params)
 
 
 def guess_filename(obj, default=None):
     name = getattr(obj, 'name', None)
-    if name and name[0] != '<' and name[-1] != '>':
+    if name and isinstance(name, str) and name[0] != '<' and name[-1] != '>':
         return Path(name).name
     return default
 
@@ -218,9 +268,10 @@ def guess_filename(obj, default=None):
 def content_disposition_header(disptype, quote_fields=True, **params):
     """Sets ``Content-Disposition`` header.
 
-    :param str disptype: Disposition type: inline, attachment, form-data.
-                         Should be valid extension token (see RFC 2183)
-    :param dict params: Disposition params
+    disptype is a disposition type: inline, attachment, form-data.
+    Should be valid extension token (see RFC 2183)
+
+    params is a dict with disposition params.
     """
     if not disptype or not (TOKEN > set(disptype)):
         raise ValueError('bad content disposition type {!r}'
@@ -242,14 +293,14 @@ def content_disposition_header(disptype, quote_fields=True, **params):
     return value
 
 
-class AccessLogger:
+class AccessLogger(AbstractAccessLogger):
     """Helper object to log access.
 
     Usage:
         log = logging.getLogger("spam")
         log_format = "%a %{User-Agent}i"
         access_logger = AccessLogger(log, log_format)
-        access_logger.log(message, environ, response, transport, time)
+        access_logger.log(request, response, time)
 
     Format:
         %%  The percent sign
@@ -270,7 +321,7 @@ class AccessLogger:
     """
     LOG_FORMAT_MAP = {
         'a': 'remote_address',
-        't': 'request_time',
+        't': 'request_start_time',
         'P': 'process_id',
         'r': 'first_request_line',
         's': 'response_status',
@@ -280,10 +331,9 @@ class AccessLogger:
         'D': 'request_time_micro',
         'i': 'request_header',
         'o': 'response_header',
-        'e': 'environ'
     }
 
-    LOG_FORMAT = '%a %l %u %t "%r" %s %b "%{Referrer}i" "%{User-Agent}i"'
+    LOG_FORMAT = '%a %t "%r" %s %b "%{Referer}i" "%{User-Agent}i"'
     FORMAT_RE = re.compile(r'%(\{([A-Za-z0-9\-_]+)\}([ioe])|[atPrsbOD]|Tf?)')
     CLEANUP_RE = re.compile(r'(%[^s])')
     _FORMAT_CACHE = {}
@@ -293,11 +343,11 @@ class AccessLogger:
     def __init__(self, logger, log_format=LOG_FORMAT):
         """Initialise the logger.
 
-        :param logger: logger object to be used for logging
-        :param log_format: apache compatible log format
+        logger is a logger object to be used for logging.
+        log_format is an string with apache compatible log format description.
 
         """
-        self.logger = logger
+        super().__init__(logger, log_format=log_format)
 
         _compiled_format = AccessLogger._FORMAT_CACHE.get(log_format)
         if not _compiled_format:
@@ -311,7 +361,7 @@ class AccessLogger:
 
         All known atoms will be replaced with %s
         Also methods for formatting of those atoms will be added to
-        _methods in apropriate order
+        _methods in appropriate order
 
         For example we have log_format = "%a %t"
         This format will be translated to "%s %s"
@@ -327,10 +377,6 @@ class AccessLogger:
         also receive key name (by functools.partial)
 
         """
-
-        log_format = log_format.replace("%l", "-")
-        log_format = log_format.replace("%u", "-")
-
         # list of (key, method) tuples, we don't use an OrderedDict as users
         # can repeat the same key more than once
         methods = list()
@@ -351,87 +397,69 @@ class AccessLogger:
         return log_format, methods
 
     @staticmethod
-    def _format_e(key, args):
-        return (args[1] or {}).get(key, '-')
-
-    @staticmethod
-    def _format_i(key, args):
-        if not args[0]:
+    def _format_i(key, request, response, time):
+        if request is None:
             return '(no headers)'
 
         # suboptimal, make istr(key) once
-        return args[0].headers.get(key, '-')
+        return request.headers.get(key, '-')
 
     @staticmethod
-    def _format_o(key, args):
+    def _format_o(key, request, response, time):
         # suboptimal, make istr(key) once
-        return args[2].headers.get(key, '-')
+        return response.headers.get(key, '-')
 
     @staticmethod
-    def _format_a(args):
-        if args[3] is None:
+    def _format_a(request, response, time):
+        if request is None:
             return '-'
-        peername = args[3].get_extra_info('peername')
-        if isinstance(peername, (list, tuple)):
-            return peername[0]
-        else:
-            return peername
+        ip = request.remote
+        return ip if ip is not None else '-'
 
     @staticmethod
-    def _format_t(args):
-        return datetime.datetime.utcnow().strftime('[%d/%b/%Y:%H:%M:%S +0000]')
+    def _format_t(request, response, time):
+        now = datetime.datetime.utcnow()
+        start_time = now - datetime.timedelta(seconds=time)
+        return start_time.strftime('[%d/%b/%Y:%H:%M:%S +0000]')
 
     @staticmethod
-    def _format_P(args):
+    def _format_P(request, response, time):
         return "<%s>" % os.getpid()
 
     @staticmethod
-    def _format_r(args):
-        msg = args[0]
-        if not msg:
+    def _format_r(request, response, time):
+        if request is None:
             return '-'
-        return '%s %s HTTP/%s.%s' % tuple((msg.method,
-                                           msg.path) + msg.version)
+        return '%s %s HTTP/%s.%s' % tuple((request.method,
+                                           request.path_qs) + request.version)
 
     @staticmethod
-    def _format_s(args):
-        return args[2].status
+    def _format_s(request, response, time):
+        return response.status
 
     @staticmethod
-    def _format_b(args):
-        return args[2].body_length
+    def _format_b(request, response, time):
+        return response.body_length
 
     @staticmethod
-    def _format_O(args):
-        return args[2].body_length
+    def _format_T(request, response, time):
+        return round(time)
 
     @staticmethod
-    def _format_T(args):
-        return round(args[4])
+    def _format_Tf(request, response, time):
+        return '%06f' % time
 
     @staticmethod
-    def _format_Tf(args):
-        return '%06f' % args[4]
+    def _format_D(request, response, time):
+        return round(time * 1000000)
 
-    @staticmethod
-    def _format_D(args):
-        return round(args[4] * 1000000)
+    def _format_line(self, request, response, time):
+        return ((key, method(request, response, time))
+                for key, method in self._methods)
 
-    def _format_line(self, args):
-        return ((key, method(args)) for key, method in self._methods)
-
-    def log(self, message, environ, response, transport, time):
-        """Log access.
-
-        :param message: Request object. May be None.
-        :param environ: Environment dict. May be None.
-        :param response: Response object.
-        :param transport: Tansport object. May be None
-        :param float time: Time taken to serve the request.
-        """
+    def log(self, request, response, time):
         try:
-            fmt_info = self._format_line(
-                [message, environ, response, transport, time])
+            fmt_info = self._format_line(request, response, time)
 
             values = list()
             extra = dict()
@@ -441,7 +469,10 @@ class AccessLogger:
                 if key.__class__ is str:
                     extra[key] = value
                 else:
-                    extra[key[0]] = {key[1]: value}
+                    k1, k2 = key
+                    dct = extra.get(k1, {})
+                    dct[k2] = value
+                    extra[k1] = dct
 
             self.logger.info(self._log_format % tuple(values), extra=extra)
         except Exception:
@@ -459,13 +490,10 @@ class reify:
 
     def __init__(self, wrapped):
         self.wrapped = wrapped
-        try:
-            self.__doc__ = wrapped.__doc__
-        except:  # pragma: no cover
-            self.__doc__ = ""
+        self.__doc__ = wrapped.__doc__
         self.name = wrapped.__name__
 
-    def __get__(self, inst, owner, _sentinel=sentinel):
+    def __get__(self, inst, owner):
         try:
             try:
                 return inst._cache[self.name]
@@ -482,6 +510,15 @@ class reify:
         raise AttributeError("reified property is read-only")
 
 
+reify_py = reify
+
+try:
+    from ._helpers import reify as reify_c
+    if not NO_EXTENSIONS:
+        reify = reify_c
+except ImportError:
+    pass
+
 _ipv4_pattern = (r'^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}'
                  r'(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$')
 _ipv6_pattern = (
@@ -517,77 +554,38 @@ def is_ip_address(host):
                         .format(host, type(host)))
 
 
-class TimeService:
+_cached_current_datetime = None
+_cached_formatted_datetime = None
 
-    def __init__(self, loop, *, interval=1.0):
-        self._loop = loop
-        self._interval = interval
-        self._time = time.time()
-        self._loop_time = loop.time()
-        self._count = 0
-        self._strtime = None
-        self._cb = loop.call_at(self._loop_time + self._interval, self._on_cb)
 
-    def close(self):
-        if self._cb:
-            self._cb.cancel()
+def rfc822_formatted_time():
+    global _cached_current_datetime
+    global _cached_formatted_datetime
 
-        self._cb = None
-        self._loop = None
-
-    def _on_cb(self, reset_count=10*60):
-        if self._count >= reset_count:
-            # reset timer every 10 minutes
-            self._count = 0
-            self._time = time.time()
-        else:
-            self._time += self._interval
-
-        self._strtime = None
-        self._loop_time = ceil(self._loop.time())
-        self._cb = self._loop.call_at(
-            self._loop_time + self._interval, self._on_cb)
-
-    def _format_date_time(self):
+    now = int(time.time())
+    if now != _cached_current_datetime:
         # Weekday and month names for HTTP date/time formatting;
         # always English!
-        # Tuples are contants stored in codeobject!
+        # Tuples are constants stored in codeobject!
         _weekdayname = ("Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun")
-        _monthname = (None,  # Dummy so we can use 1-based month numbers
+        _monthname = ("",  # Dummy so we can use 1-based month numbers
                       "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                       "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
 
-        year, month, day, hh, mm, ss, wd, y, z = gmtime(self._time)
-        return "%s, %02d %3s %4d %02d:%02d:%02d GMT" % (
+        year, month, day, hh, mm, ss, wd, y, z = time.gmtime(now)
+        _cached_formatted_datetime = "%s, %02d %3s %4d %02d:%02d:%02d GMT" % (
             _weekdayname[wd], day, _monthname[month], year, hh, mm, ss
         )
-
-    def time(self):
-        return self._time
-
-    def strtime(self):
-        s = self._strtime
-        if s is None:
-            self._strtime = s = self._format_date_time()
-        return self._strtime
-
-    @property
-    def loop_time(self):
-        return self._loop_time
-
-    @property
-    def interval(self):
-        return self._interval
+        _cached_current_datetime = now
+    return _cached_formatted_datetime
 
 
 def _weakref_handle(info):
     ref, name = info
     ob = ref()
     if ob is not None:
-        try:
+        with suppress(Exception):
             getattr(ob, name)()
-        except:
-            pass
 
 
 def weakref_handle(ob, name, timeout, loop, ceil_timeout=True):
@@ -634,10 +632,8 @@ class TimeoutHandle:
 
     def __call__(self):
         for cb, args, kwargs in self._callbacks:
-            try:
+            with suppress(Exception):
                 cb(*args, **kwargs)
-            except:
-                pass
 
         self._callbacks.clear()
 
@@ -688,7 +684,7 @@ class TimerContext:
             self._cancelled = True
 
 
-class CeilTimeout(Timeout):
+class CeilTimeout(async_timeout.timeout):
 
     def __enter__(self):
         if self._timeout is not None:
@@ -703,6 +699,9 @@ class CeilTimeout(Timeout):
 
 class HeadersMixin:
 
+    ATTRS = frozenset([
+        '_content_type', '_content_dict', '_stored_content_type'])
+
     _content_type = None
     _content_dict = None
     _stored_content_type = sentinel
@@ -735,35 +734,60 @@ class HeadersMixin:
     @property
     def content_length(self, *, _CONTENT_LENGTH=hdrs.CONTENT_LENGTH):
         """The value of Content-Length HTTP header."""
-        l = self._headers.get(_CONTENT_LENGTH)
-        if l is None:
-            return None
-        else:
-            return int(l)
+        content_length = self._headers.get(_CONTENT_LENGTH)
+
+        if content_length:
+            return int(content_length)
 
 
-class DummyCookieJar(AbstractCookieJar):
-    """Implements a dummy cookie storage.
+def set_result(fut, result):
+    if not fut.done():
+        fut.set_result(result)
 
-    It can be used with the ClientSession when no cookie processing is needed.
 
-    """
+def set_exception(fut, exc):
+    if not fut.done():
+        fut.set_exception(exc)
 
-    def __init__(self, *, loop=None):
-        super().__init__(loop=loop)
 
-    def __iter__(self):
-        while False:
-            yield None
+class ChainMapProxy(Mapping):
+    __slots__ = ('_maps',)
+
+    def __init__(self, maps):
+        self._maps = tuple(maps)
+
+    def __init_subclass__(cls):
+        raise TypeError("Inheritance class {} from ChainMapProxy "
+                        "is forbidden".format(cls.__name__))
+
+    def __getitem__(self, key):
+        for mapping in self._maps:
+            try:
+                return mapping[key]
+            except KeyError:
+                pass
+        raise KeyError(key)
+
+    def get(self, key, default=None):
+        return self[key] if key in self else default
 
     def __len__(self):
-        return 0
+        # reuses stored hash values if possible
+        return len(set().union(*self._maps))
 
-    def clear(self):
-        pass
+    def __iter__(self):
+        d = {}
+        for mapping in reversed(self._maps):
+            # reuses stored hash values if possible
+            d.update(mapping)
+        return iter(d)
 
-    def update_cookies(self, cookies, response_url=None):
-        pass
+    def __contains__(self, key):
+        return any(key in m for m in self._maps)
 
-    def filter_cookies(self, request_url):
-        return None
+    def __bool__(self):
+        return any(self._maps)
+
+    def __repr__(self):
+        content = ", ".join(map(repr, self._maps))
+        return 'ChainMapProxy({})'.format(content)
