@@ -4,8 +4,8 @@ import string
 import zlib
 from enum import IntEnum
 
-import yarl
-from multidict import CIMultiDict, istr
+from multidict import CIMultiDict, CIMultiDictProxy
+from yarl import URL
 
 from . import hdrs
 from .helpers import NO_EXTENSIONS
@@ -14,7 +14,15 @@ from .http_exceptions import (BadStatusLine, ContentEncodingError,
                               TransferEncodingError)
 from .http_writer import HttpVersion, HttpVersion10
 from .log import internal_logger
-from .streams import EMPTY_PAYLOAD, FlowControlStreamReader
+from .streams import EMPTY_PAYLOAD, StreamReader
+
+
+try:
+    import brotli
+    HAS_BROTLI = True
+except ImportError:  # pragma: no cover
+    HAS_BROTLI = False
+
 
 __all__ = (
     'HttpParser', 'HttpRequestParser', 'HttpResponseParser',
@@ -58,7 +66,8 @@ class HttpParser:
                  max_line_size=8190, max_headers=32768, max_field_size=8190,
                  timer=None, code=None, method=None, readall=False,
                  payload_exception=None,
-                 response_with_body=True, read_until_eof=False):
+                 response_with_body=True, read_until_eof=False,
+                 auto_decompress=True):
         self.protocol = protocol
         self.loop = loop
         self.max_line_size = max_line_size
@@ -77,11 +86,24 @@ class HttpParser:
         self._upgraded = False
         self._payload = None
         self._payload_parser = None
+        self._auto_decompress = auto_decompress
 
     def feed_eof(self):
         if self._payload_parser is not None:
             self._payload_parser.feed_eof()
             self._payload_parser = None
+        else:
+            # try to extract partial message
+            if self._tail:
+                self._lines.append(self._tail)
+
+            if self._lines:
+                if self._lines[-1] != '\r\n':
+                    self._lines.append('')
+                try:
+                    return self.parse_message(self._lines)
+                except Exception:
+                    return None
 
     def feed_data(self, data,
                   SEP=b'\r\n', EMPTY=b'',
@@ -104,6 +126,11 @@ class HttpParser:
             # and split by lines
             if self._payload_parser is None and not self._upgraded:
                 pos = data.find(SEP, start_pos)
+                # consume \r\n
+                if pos == start_pos and not self._lines:
+                    start_pos = pos + 2
+                    continue
+
                 if pos >= start_pos:
                     # line found
                     self._lines.append(data[start_pos:pos])
@@ -137,34 +164,37 @@ class HttpParser:
                         # calculate payload
                         if ((length is not None and length > 0) or
                                 msg.chunked and not msg.upgrade):
-                            payload = FlowControlStreamReader(
+                            payload = StreamReader(
                                 self.protocol, timer=self.timer, loop=loop)
                             payload_parser = HttpPayloadParser(
                                 payload, length=length,
                                 chunked=msg.chunked, method=method,
                                 compression=msg.compression,
                                 code=self.code, readall=self.readall,
-                                response_with_body=self.response_with_body)
+                                response_with_body=self.response_with_body,
+                                auto_decompress=self._auto_decompress)
                             if not payload_parser.done:
                                 self._payload_parser = payload_parser
                         elif method == METH_CONNECT:
-                            payload = FlowControlStreamReader(
+                            payload = StreamReader(
                                 self.protocol, timer=self.timer, loop=loop)
                             self._upgraded = True
                             self._payload_parser = HttpPayloadParser(
                                 payload, method=msg.method,
-                                compression=msg.compression, readall=True)
+                                compression=msg.compression, readall=True,
+                                auto_decompress=self._auto_decompress)
                         else:
                             if (getattr(msg, 'code', 100) >= 199 and
                                     length is None and self.read_until_eof):
-                                payload = FlowControlStreamReader(
+                                payload = StreamReader(
                                     self.protocol, timer=self.timer, loop=loop)
                                 payload_parser = HttpPayloadParser(
                                     payload, length=length,
                                     chunked=msg.chunked, method=method,
                                     compression=msg.compression,
                                     code=self.code, readall=True,
-                                    response_with_body=self.response_with_body)
+                                    response_with_body=self.response_with_body,
+                                    auto_decompress=self._auto_decompress)
                                 if not payload_parser.done:
                                     self._payload_parser = payload_parser
                             else:
@@ -226,8 +256,6 @@ class HttpParser:
         line_count = len(lines)
 
         while line:
-            header_length = len(line)
-
             # Parse initial header name : value pair.
             try:
                 bname, bvalue = line.split(b':', 1)
@@ -235,8 +263,17 @@ class HttpParser:
                 raise InvalidHeader(line) from None
 
             bname = bname.strip(b' \t')
+            bvalue = bvalue.lstrip()
             if HDRRE.search(bname):
                 raise InvalidHeader(bname)
+            if len(bname) > self.max_field_size:
+                raise LineTooLong(
+                    "request header name {}".format(
+                        bname.decode("utf8", "xmlcharrefreplace")),
+                    self.max_field_size,
+                    len(bname))
+
+            header_length = len(bvalue)
 
             # next line
             lines_idx += 1
@@ -253,7 +290,8 @@ class HttpParser:
                         raise LineTooLong(
                             'request header field {}'.format(
                                 bname.decode("utf8", "xmlcharrefreplace")),
-                            self.max_field_size)
+                            self.max_field_size,
+                            header_length)
                     bvalue.append(line)
 
                     # next line
@@ -271,10 +309,11 @@ class HttpParser:
                     raise LineTooLong(
                         'request header field {}'.format(
                             bname.decode("utf8", "xmlcharrefreplace")),
-                        self.max_field_size)
+                        self.max_field_size,
+                        header_length)
 
             bvalue = bvalue.strip()
-            name = istr(bname.decode('utf-8', 'surrogateescape'))
+            name = bname.decode('utf-8', 'surrogateescape')
             value = bvalue.decode('utf-8', 'surrogateescape')
 
             headers.add(name, value)
@@ -285,6 +324,7 @@ class HttpParser:
         upgrade = False
         chunked = False
         raw_headers = tuple(raw_headers)
+        headers = CIMultiDictProxy(headers)
 
         # keep-alive
         conn = headers.get(hdrs.CONNECTION)
@@ -301,7 +341,7 @@ class HttpParser:
         enc = headers.get(hdrs.CONTENT_ENCODING)
         if enc:
             enc = enc.lower()
-            if enc in ('gzip', 'deflate'):
+            if enc in ('gzip', 'deflate', 'br'):
                 encoding = enc
 
         # chunking
@@ -319,10 +359,6 @@ class HttpRequestParserPy(HttpParser):
     """
 
     def parse_message(self, lines):
-        if len(lines[0]) > self.max_line_size:
-            raise LineTooLong(
-                'Status line is too long', self.max_line_size)
-
         # request line
         line = lines[0].decode('utf-8', 'surrogateescape')
         try:
@@ -330,6 +366,10 @@ class HttpRequestParserPy(HttpParser):
         except ValueError:
             raise BadStatusLine(line) from None
 
+        if len(path) > self.max_line_size:
+            raise LineTooLong(
+                'Status line is too long', self.max_line_size, len(path))
+
         # method
         method = method.upper()
         if not METHRE.match(method):
@@ -342,12 +382,12 @@ class HttpRequestParserPy(HttpParser):
                 version = HttpVersion(int(n1), int(n2))
             else:
                 raise BadStatusLine(version)
-        except:
+        except Exception:
             raise BadStatusLine(version)
 
         # read headers
-        headers, raw_headers, \
-            close, compression, upgrade, chunked = self.parse_headers(lines)
+        (headers, raw_headers,
+         close, compression, upgrade, chunked) = self.parse_headers(lines)
 
         if close is None:  # then the headers weren't set in the request
             if version <= HttpVersion10:  # HTTP 1.0 must asks to not close
@@ -357,7 +397,7 @@ class HttpRequestParserPy(HttpParser):
 
         return RawRequestMessage(
             method, path, version, headers, raw_headers,
-            close, compression, upgrade, chunked, yarl.URL(path))
+            close, compression, upgrade, chunked, URL(path))
 
 
 class HttpResponseParserPy(HttpParser):
@@ -367,20 +407,21 @@ class HttpResponseParserPy(HttpParser):
     Returns RawResponseMessage"""
 
     def parse_message(self, lines):
-        if len(lines[0]) > self.max_line_size:
-            raise LineTooLong(
-                'Status line is too long', self.max_line_size)
-
         line = lines[0].decode('utf-8', 'surrogateescape')
         try:
             version, status = line.split(None, 1)
         except ValueError:
             raise BadStatusLine(line) from None
-        else:
-            try:
-                status, reason = status.split(None, 1)
-            except ValueError:
-                reason = ''
+
+        try:
+            status, reason = status.split(None, 1)
+        except ValueError:
+            reason = ''
+
+        if len(reason) > self.max_line_size:
+            raise LineTooLong(
+                'Status line is too long', self.max_line_size,
+                len(reason))
 
         # version
         match = VERSRE.match(version)
@@ -398,8 +439,8 @@ class HttpResponseParserPy(HttpParser):
             raise BadStatusLine(line)
 
         # read headers
-        headers, raw_headers, \
-            close, compression, upgrade, chunked = self.parse_headers(lines)
+        (headers, raw_headers,
+         close, compression, upgrade, chunked) = self.parse_headers(lines)
 
         if close is None:
             close = version <= HttpVersion10
@@ -414,7 +455,7 @@ class HttpPayloadParser:
     def __init__(self, payload,
                  length=None, chunked=False, compression=None,
                  code=None, method=None,
-                 readall=False, response_with_body=True):
+                 readall=False, response_with_body=True, auto_decompress=True):
         self.payload = payload
 
         self._length = 0
@@ -422,10 +463,11 @@ class HttpPayloadParser:
         self._chunk = ChunkState.PARSE_CHUNKED_SIZE
         self._chunk_size = 0
         self._chunk_tail = b''
+        self._auto_decompress = auto_decompress
         self.done = False
 
         # payload decompression wrapper
-        if (response_with_body and compression):
+        if response_with_body and compression and self._auto_decompress:
             payload = DeflateBuffer(payload, compression)
 
         # payload parser
@@ -502,7 +544,7 @@ class HttpPayloadParser:
                             size = chunk[:pos]
 
                         try:
-                            size = int(size, 16)
+                            size = int(bytes(size), 16)
                         except ValueError:
                             exc = TransferEncodingError(chunk[:pos])
                             self.payload.set_exception(exc)
@@ -514,6 +556,7 @@ class HttpPayloadParser:
                         else:
                             self._chunk = ChunkState.PARSE_CHUNKED_CHUNK
                             self._chunk_size = size
+                            self.payload.begin_http_chunk_receiving()
                     else:
                         self._chunk_tail = chunk
                         return False, None
@@ -523,11 +566,8 @@ class HttpPayloadParser:
                     required = self._chunk_size
                     chunk_len = len(chunk)
 
-                    if required >= chunk_len:
+                    if required > chunk_len:
                         self._chunk_size = required - chunk_len
-                        if self._chunk_size == 0:
-                            self._chunk = ChunkState.PARSE_CHUNKED_CHUNK_EOF
-
                         self.payload.feed_data(chunk, chunk_len)
                         return False, None
                     else:
@@ -535,6 +575,7 @@ class HttpPayloadParser:
                         self.payload.feed_data(chunk[:required], required)
                         chunk = chunk[required:]
                         self._chunk = ChunkState.PARSE_CHUNKED_CHUNK_EOF
+                        self.payload.end_http_chunk_receiving()
 
                 # toss the CRLF at the end of the chunk
                 if self._chunk == ChunkState.PARSE_CHUNKED_CHUNK_EOF:
@@ -580,11 +621,18 @@ class DeflateBuffer:
         self.out = out
         self.size = 0
         self.encoding = encoding
+        self._started_decoding = False
 
-        zlib_mode = (16 + zlib.MAX_WBITS
-                     if encoding == 'gzip' else -zlib.MAX_WBITS)
-
-        self.zlib = zlib.decompressobj(wbits=zlib_mode)
+        if encoding == 'br':
+            if not HAS_BROTLI:  # pragma: no cover
+                raise ContentEncodingError(
+                    'Can not decode content-encoding: brotli (br). '
+                    'Please install `brotlipy`')
+            self.decompressor = brotli.Decompressor()
+        else:
+            zlib_mode = (16 + zlib.MAX_WBITS
+                         if encoding == 'gzip' else -zlib.MAX_WBITS)
+            self.decompressor = zlib.decompressobj(wbits=zlib_mode)
 
     def set_exception(self, exc):
         self.out.set_exception(exc)
@@ -592,24 +640,39 @@ class DeflateBuffer:
     def feed_data(self, chunk, size):
         self.size += size
         try:
-            chunk = self.zlib.decompress(chunk)
+            chunk = self.decompressor.decompress(chunk)
         except Exception:
-            raise ContentEncodingError(
-                'Can not decode content-encoding: %s' % self.encoding)
+            if not self._started_decoding and self.encoding == 'deflate':
+                self.decompressor = zlib.decompressobj()
+                try:
+                    chunk = self.decompressor.decompress(chunk)
+                except Exception:
+                    raise ContentEncodingError(
+                        'Can not decode content-encoding: %s' % self.encoding)
+            else:
+                raise ContentEncodingError(
+                    'Can not decode content-encoding: %s' % self.encoding)
 
         if chunk:
+            self._started_decoding = True
             self.out.feed_data(chunk, len(chunk))
 
     def feed_eof(self):
-        chunk = self.zlib.flush()
+        chunk = self.decompressor.flush()
 
         if chunk or self.size > 0:
             self.out.feed_data(chunk, len(chunk))
-            if not self.zlib.eof:
+            if self.encoding != 'br' and not self.decompressor.eof:
                 raise ContentEncodingError('deflate')
 
         self.out.feed_eof()
 
+    def begin_http_chunk_receiving(self):
+        self.out.begin_http_chunk_receiving()
+
+    def end_http_chunk_receiving(self):
+        self.out.end_http_chunk_receiving()
+
 
 HttpRequestParser = HttpRequestParserPy
 HttpResponseParser = HttpResponseParserPy
