@@ -5,30 +5,23 @@ import contextlib
 import functools
 import gc
 import socket
+import sys
 import unittest
 from abc import ABC, abstractmethod
-from contextlib import contextmanager
 from unittest import mock
 
-from multidict import CIMultiDict
+from multidict import CIMultiDict, CIMultiDictProxy
 from yarl import URL
 
 import aiohttp
-from aiohttp.client import _RequestContextManager
+from aiohttp.client import _RequestContextManager, _WSRequestContextManager
 
 from . import ClientSession, hdrs
-from .helpers import PY_35, noop, sentinel
+from .helpers import sentinel
 from .http import HttpVersion, RawRequestMessage
 from .signals import Signal
-from .web import Application, Request, Server, UrlMappingMatchInfo
-
-
-def run_briefly(loop):
-    @asyncio.coroutine
-    def once():
-        pass
-    t = asyncio.Task(once(), loop=loop)
-    loop.run_until_complete(t)
+from .web import (AppRunner, Request, Server, ServerRunner, TCPSite,
+                  UrlMappingMatchInfo)
 
 
 def unused_port():
@@ -40,24 +33,29 @@ def unused_port():
 
 class BaseTestServer(ABC):
     def __init__(self, *, scheme=sentinel, loop=None,
-                 host='127.0.0.1', skip_url_asserts=False, **kwargs):
+                 host='127.0.0.1', port=None, skip_url_asserts=False,
+                 **kwargs):
         self._loop = loop
-        self.port = None
-        self.server = None
-        self.handler = None
+        self.runner = None
         self._root = None
         self.host = host
+        self.port = port
         self._closed = False
         self.scheme = scheme
         self.skip_url_asserts = skip_url_asserts
 
-    @asyncio.coroutine
-    def start_server(self, loop=None, **kwargs):
-        if self.server:
+    async def start_server(self, loop=None, **kwargs):
+        if self.runner:
             return
         self._loop = loop
-        self.port = unused_port()
         self._ssl = kwargs.pop('ssl', None)
+        self.runner = await self._make_runner(**kwargs)
+        await self.runner.setup()
+        if not self.port:
+            self.port = unused_port()
+        site = TCPSite(self.runner, host=self.host, port=self.port,
+                       ssl_context=self._ssl)
+        await site.start()
         if self.scheme is sentinel:
             if self._ssl:
                 scheme = 'https'
@@ -68,13 +66,8 @@ class BaseTestServer(ABC):
                                              self.host,
                                              self.port))
 
-        handler = yield from self._make_factory(**kwargs)
-        self.server = yield from self._loop.create_server(
-            handler, self.host, self.port, ssl=self._ssl)
-
     @abstractmethod  # pragma: no cover
-    @asyncio.coroutine
-    def _make_factory(self, **kwargs):
+    async def _make_runner(self, **kwargs):
         pass
 
     def make_url(self, path):
@@ -87,14 +80,19 @@ class BaseTestServer(ABC):
 
     @property
     def started(self):
-        return self.server is not None
+        return self.runner is not None
 
     @property
     def closed(self):
         return self._closed
 
-    @asyncio.coroutine
-    def close(self):
+    @property
+    def handler(self):
+        # for backward compatibility
+        # web.Server instance
+        return self.runner.server
+
+    async def close(self):
         """Close all fixtures created by the test client.
 
         After that point, the TestClient is no longer usable.
@@ -107,72 +105,48 @@ class BaseTestServer(ABC):
 
         """
         if self.started and not self.closed:
-            self.server.close()
-            yield from self.server.wait_closed()
+            await self.runner.cleanup()
             self._root = None
             self.port = None
-            yield from self._close_hook()
             self._closed = True
 
-    @abstractmethod
-    @asyncio.coroutine
-    def _close_hook(self):
-        pass  # pragma: no cover
-
     def __enter__(self):
-        self._loop.run_until_complete(self.start_server(loop=self._loop))
-        return self
+        raise TypeError("Use async with instead")
 
     def __exit__(self, exc_type, exc_value, traceback):
-        self._loop.run_until_complete(self.close())
+        # __exit__ should exist in pair with __enter__ but never executed
+        pass  # pragma: no cover
 
-    if PY_35:
-        @asyncio.coroutine
-        def __aenter__(self):
-            yield from self.start_server(loop=self._loop)
-            return self
+    async def __aenter__(self):
+        await self.start_server(loop=self._loop)
+        return self
 
-        @asyncio.coroutine
-        def __aexit__(self, exc_type, exc_value, traceback):
-            yield from self.close()
+    async def __aexit__(self, exc_type, exc_value, traceback):
+        await self.close()
 
 
 class TestServer(BaseTestServer):
 
     def __init__(self, app, *,
-                 scheme=sentinel, host='127.0.0.1', **kwargs):
+                 scheme=sentinel, host='127.0.0.1', port=None, **kwargs):
         self.app = app
-        super().__init__(scheme=scheme, host=host, **kwargs)
+        super().__init__(scheme=scheme, host=host, port=port, **kwargs)
 
-    @asyncio.coroutine
-    def _make_factory(self, **kwargs):
-        self.handler = self.app.make_handler(loop=self._loop, **kwargs)
-        yield from self.app.startup()
-        return self.handler
-
-    @asyncio.coroutine
-    def _close_hook(self):
-        yield from self.app.shutdown()
-        yield from self.handler.shutdown()
-        yield from self.app.cleanup()
+    async def _make_runner(self, **kwargs):
+        return AppRunner(self.app, **kwargs)
 
 
 class RawTestServer(BaseTestServer):
 
     def __init__(self, handler, *,
-                 scheme=sentinel, host='127.0.0.1', **kwargs):
+                 scheme=sentinel, host='127.0.0.1', port=None, **kwargs):
         self._handler = handler
-        super().__init__(scheme=scheme, host=host, **kwargs)
+        super().__init__(scheme=scheme, host=host, port=port, **kwargs)
 
-    @asyncio.coroutine
-    def _make_factory(self, debug=True, **kwargs):
-        self.handler = Server(
+    async def _make_runner(self, debug=True, **kwargs):
+        srv = Server(
             self._handler, loop=self._loop, debug=True, **kwargs)
-        return self.handler
-
-    @asyncio.coroutine
-    def _close_hook(self):
-        return
+        return ServerRunner(srv, debug=debug, **kwargs)
 
 
 class TestClient:
@@ -183,23 +157,11 @@ class TestClient:
 
     """
 
-    def __init__(self, app_or_server, *, scheme=sentinel, host=sentinel,
-                 cookie_jar=None, server_kwargs=None, loop=None, **kwargs):
-        if isinstance(app_or_server, BaseTestServer):
-            if scheme is not sentinel or host is not sentinel:
-                raise ValueError("scheme and host are mutable exclusive "
-                                 "with TestServer parameter")
-            self._server = app_or_server
-        elif isinstance(app_or_server, Application):
-            scheme = "http" if scheme is sentinel else scheme
-            host = '127.0.0.1' if host is sentinel else host
-            server_kwargs = server_kwargs or {}
-            self._server = TestServer(
-                app_or_server,
-                scheme=scheme, host=host, **server_kwargs)
-        else:
-            raise TypeError("app_or_server should be either web.Application "
-                            "or TestServer instance")
+    def __init__(self, server, *, cookie_jar=None, loop=None, **kwargs):
+        if not isinstance(server, BaseTestServer):
+            raise TypeError("server must be TestServer "
+                            "instance, found type: %r" % type(server))
+        self._server = server
         self._loop = loop
         if cookie_jar is None:
             cookie_jar = aiohttp.CookieJar(unsafe=True, loop=loop)
@@ -210,9 +172,8 @@ class TestClient:
         self._responses = []
         self._websockets = []
 
-    @asyncio.coroutine
-    def start_server(self):
-        yield from self._server.start_server(loop=self._loop)
+    async def start_server(self):
+        await self._server.start_server(loop=self._loop)
 
     @property
     def host(self):
@@ -226,6 +187,10 @@ class TestClient:
     def server(self):
         return self._server
 
+    @property
+    def app(self):
+        return getattr(self._server, "app", None)
+
     @property
     def session(self):
         """An internal aiohttp.ClientSession.
@@ -240,8 +205,7 @@ class TestClient:
     def make_url(self, path):
         return self._server.make_url(path)
 
-    @asyncio.coroutine
-    def request(self, method, path, *args, **kwargs):
+    async def request(self, method, path, *args, **kwargs):
         """Routes a request to tested http server.
 
         The interface is identical to asyncio.ClientSession.request,
@@ -249,7 +213,7 @@ class TestClient:
         test server.
 
         """
-        resp = yield from self._session.request(
+        resp = await self._session.request(
             method, self.make_url(path), *args, **kwargs
         )
         # save it to close later
@@ -298,20 +262,23 @@ class TestClient:
             self.request(hdrs.METH_DELETE, path, *args, **kwargs)
         )
 
-    @asyncio.coroutine
     def ws_connect(self, path, *args, **kwargs):
         """Initiate websocket connection.
 
         The api corresponds to aiohttp.ClientSession.ws_connect.
 
         """
-        ws = yield from self._session.ws_connect(
+        return _WSRequestContextManager(
+            self._ws_connect(path, *args, **kwargs)
+        )
+
+    async def _ws_connect(self, path, *args, **kwargs):
+        ws = await self._session.ws_connect(
             self.make_url(path), *args, **kwargs)
         self._websockets.append(ws)
         return ws
 
-    @asyncio.coroutine
-    def close(self):
+    async def close(self):
         """Close all fixtures created by the test client.
 
         After that point, the TestClient is no longer usable.
@@ -327,27 +294,24 @@ class TestClient:
             for resp in self._responses:
                 resp.close()
             for ws in self._websockets:
-                yield from ws.close()
-            self._session.close()
-            yield from self._server.close()
+                await ws.close()
+            await self._session.close()
+            await self._server.close()
             self._closed = True
 
     def __enter__(self):
-        self._loop.run_until_complete(self.start_server())
-        return self
+        raise TypeError("Use async with instead")
 
     def __exit__(self, exc_type, exc_value, traceback):
-        self._loop.run_until_complete(self.close())
+        # __exit__ should exist in pair with __enter__ but never executed
+        pass  # pragma: no cover
 
-    if PY_35:
-        @asyncio.coroutine
-        def __aenter__(self):
-            yield from self.start_server()
-            return self
+    async def __aenter__(self):
+        await self.start_server()
+        return self
 
-        @asyncio.coroutine
-        def __aexit__(self, exc_type, exc_value, traceback):
-            yield from self.close()
+    async def __aexit__(self, exc_type, exc_value, traceback):
+        await self.close()
 
 
 class AioHTTPTestCase(unittest.TestCase):
@@ -366,8 +330,7 @@ class AioHTTPTestCase(unittest.TestCase):
     execute function on the test client using asynchronous methods.
     """
 
-    @asyncio.coroutine
-    def get_application(self):
+    async def get_application(self):
         """
         This method should be overridden
         to return the aiohttp.web.Application
@@ -382,27 +345,41 @@ class AioHTTPTestCase(unittest.TestCase):
         Use .get_application() coroutine instead
 
         """
-        pass  # pragma: no cover
+        raise RuntimeError("Did you forget to define get_application()?")
 
     def setUp(self):
         self.loop = setup_test_loop()
 
         self.app = self.loop.run_until_complete(self.get_application())
-        self.client = self.loop.run_until_complete(self._get_client(self.app))
+        self.server = self.loop.run_until_complete(self.get_server(self.app))
+        self.client = self.loop.run_until_complete(
+            self.get_client(self.server))
 
         self.loop.run_until_complete(self.client.start_server())
 
+        self.loop.run_until_complete(self.setUpAsync())
+
+    async def setUpAsync(self):
+        pass
+
     def tearDown(self):
+        self.loop.run_until_complete(self.tearDownAsync())
         self.loop.run_until_complete(self.client.close())
         teardown_test_loop(self.loop)
 
-    @asyncio.coroutine
-    def _get_client(self, app):
+    async def tearDownAsync(self):
+        pass
+
+    async def get_server(self, app):
+        """Return a TestServer instance."""
+        return TestServer(app, loop=self.loop)
+
+    async def get_client(self, server):
         """Return a TestClient instance."""
-        return TestClient(self.app, loop=self.loop)
+        return TestClient(server, loop=self.loop)
 
 
-def unittest_run_loop(func):
+def unittest_run_loop(func, *args, **kwargs):
     """A decorator dedicated to use with asynchronous methods of an
     AioHTTPTestCase.
 
@@ -410,9 +387,10 @@ def unittest_run_loop(func):
     the self.loop of the AioHTTPTestCase.
     """
 
-    @functools.wraps(func)
-    def new_func(self):
-        return self.loop.run_until_complete(func(self))
+    @functools.wraps(func, *args, **kwargs)
+    def new_func(self, *inner_args, **inner_kwargs):
+        return self.loop.run_until_complete(
+            func(self, *inner_args, **inner_kwargs))
 
     return new_func
 
@@ -436,7 +414,19 @@ def setup_test_loop(loop_factory=asyncio.new_event_loop):
     once they are done with the loop.
     """
     loop = loop_factory()
-    asyncio.set_event_loop(None)
+    try:
+        module = loop.__class__.__module__
+        skip_watcher = 'uvloop' in module
+    except AttributeError:  # pragma: no cover
+        # Just in case
+        skip_watcher = True
+    asyncio.set_event_loop(loop)
+    if sys.platform != "win32" and not skip_watcher:
+        policy = asyncio.get_event_loop_policy()
+        watcher = asyncio.SafeChildWatcher()
+        watcher.attach_loop(loop)
+        with contextlib.suppress(NotImplementedError):
+            policy.set_child_watcher(watcher)
     return loop
 
 
@@ -461,6 +451,7 @@ def _create_app_mock():
     app = mock.Mock()
     app._debug = False
     app.on_response_prepare = Signal(app)
+    app.on_response_prepare.freeze()
     return app
 
 
@@ -478,16 +469,16 @@ def _create_transport(sslcontext=None):
 
 
 def make_mocked_request(method, path, headers=None, *,
+                        match_info=sentinel,
                         version=HttpVersion(1, 1), closing=False,
                         app=None,
                         writer=sentinel,
-                        payload_writer=sentinel,
                         protocol=sentinel,
                         transport=sentinel,
                         payload=sentinel,
                         sslcontext=None,
-                        secure_proxy_ssl_header=None,
-                        client_max_size=1024**2):
+                        client_max_size=1024**2,
+                        loop=...):
     """Creates mocked web.Request testing purposes.
 
     Useful in unit tests, when spinning full web server is overkill or
@@ -496,18 +487,19 @@ def make_mocked_request(method, path, headers=None, *,
     """
 
     task = mock.Mock()
-    loop = mock.Mock()
-    loop.create_future.return_value = ()
+    if loop is ...:
+        loop = mock.Mock()
+        loop.create_future.return_value = ()
 
     if version < HttpVersion(1, 1):
         closing = True
 
     if headers:
-        headers = CIMultiDict(headers)
+        headers = CIMultiDictProxy(CIMultiDict(headers))
         raw_hdrs = tuple(
             (k.encode('utf-8'), v.encode('utf-8')) for k, v in headers.items())
     else:
-        headers = CIMultiDict()
+        headers = CIMultiDictProxy(CIMultiDict())
         raw_hdrs = ()
 
     chunked = 'chunked' in headers.get(hdrs.TRANSFER_ENCODING, '').lower()
@@ -518,44 +510,33 @@ def make_mocked_request(method, path, headers=None, *,
     if app is None:
         app = _create_app_mock()
 
-    if protocol is sentinel:
-        protocol = mock.Mock()
-
     if transport is sentinel:
         transport = _create_transport(sslcontext)
 
+    if protocol is sentinel:
+        protocol = mock.Mock()
+        protocol.transport = transport
+
     if writer is sentinel:
         writer = mock.Mock()
+        writer.write_headers = make_mocked_coro(None)
+        writer.write = make_mocked_coro(None)
+        writer.write_eof = make_mocked_coro(None)
+        writer.drain = make_mocked_coro(None)
         writer.transport = transport
 
-    if payload_writer is sentinel:
-        payload_writer = mock.Mock()
-        payload_writer.write_eof.side_effect = noop
-        payload_writer.drain.side_effect = noop
-
     protocol.transport = transport
     protocol.writer = writer
 
     if payload is sentinel:
         payload = mock.Mock()
 
-    time_service = mock.Mock()
-    time_service.time.return_value = 12345
-    time_service.strtime.return_value = "Tue, 15 Nov 1994 08:12:31 GMT"
-
-    @contextmanager
-    def timeout(*args, **kw):
-        yield
-
-    time_service.timeout = mock.Mock()
-    time_service.timeout.side_effect = timeout
-
     req = Request(message, payload,
-                  protocol, payload_writer, time_service, task,
-                  secure_proxy_ssl_header=secure_proxy_ssl_header,
+                  protocol, writer, task, loop,
                   client_max_size=client_max_size)
 
-    match_info = UrlMappingMatchInfo({}, mock.Mock())
+    match_info = UrlMappingMatchInfo(
+        {} if match_info is sentinel else match_info, mock.Mock())
     match_info.add_app(app)
     req._match_info = match_info
 
