@@ -1,43 +1,42 @@
 import asyncio
 import asyncio.streams
 import http.server
-import socket
 import traceback
 import warnings
 from collections import deque
 from contextlib import suppress
 from html import escape as html_escape
 
+import yarl
+
 from . import helpers, http
-from .helpers import CeilTimeout, create_future, ensure_future
-from .http import (HttpProcessingError, HttpRequestParser, PayloadWriter,
-                   StreamWriter)
+from .base_protocol import BaseProtocol
+from .helpers import CeilTimeout
+from .http import HttpProcessingError, HttpRequestParser, StreamWriter
 from .log import access_logger, server_logger
 from .streams import EMPTY_PAYLOAD
+from .tcp_helpers import tcp_cork, tcp_keepalive, tcp_nodelay
 from .web_exceptions import HTTPException
 from .web_request import BaseRequest
 from .web_response import Response
 
-__all__ = ('RequestHandler', 'RequestPayloadError')
+
+__all__ = ('RequestHandler', 'RequestPayloadError', 'PayloadAccessError')
 
 ERROR = http.RawRequestMessage(
     'UNKNOWN', '/', http.HttpVersion10, {},
-    {}, True, False, False, False, http.URL('/'))
-
-if hasattr(socket, 'SO_KEEPALIVE'):
-    def tcp_keepalive(server, transport):
-        sock = transport.get_extra_info('socket')
-        sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
-else:
-    def tcp_keepalive(server, transport):  # pragma: no cover
-        pass
+    {}, True, False, False, False, yarl.URL('/'))
 
 
 class RequestPayloadError(Exception):
     """Payload parsing error."""
 
 
-class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
+class PayloadAccessError(Exception):
+    """Payload was accesed after responce was sent."""
+
+
+class RequestHandler(BaseProtocol):
     """HTTP protocol implementation.
 
     RequestHandler handles incoming HTTP request. It reads request line,
@@ -48,8 +47,6 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
     status line, bad headers or incomplete payload. If any error occurs,
     connection gets closed.
 
-    :param time_service: Low resolution time service
-
     :param keepalive_timeout: number of seconds before closing
                               keep-alive connection
     :type keepalive_timeout: int or None
@@ -61,6 +58,9 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
     :param logger: custom logger object
     :type logger: aiohttp.log.server_logger
 
+    :param access_log_class: custom class for access_logger
+    :type access_log_class: aiohttp.abc.AbstractAccessLogger
+
     :param access_log: custom logging object
     :type access_log: aiohttp.log.server_logger
 
@@ -77,35 +77,24 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
     """
     _request_count = 0
     _keepalive = False  # keep transport open
+    KEEPALIVE_RESCHEDULE_DELAY = 1
 
     def __init__(self, manager, *, loop=None,
                  keepalive_timeout=75,  # NGINX default value is 75 secs
                  tcp_keepalive=True,
-                 slow_request_timeout=None,
                  logger=server_logger,
+                 access_log_class=helpers.AccessLogger,
                  access_log=access_logger,
                  access_log_format=helpers.AccessLogger.LOG_FORMAT,
                  debug=False,
                  max_line_size=8190,
                  max_headers=32768,
                  max_field_size=8190,
-                 lingering_time=10.0,
-                 max_concurrent_handlers=2,
-                 **kwargs):
-
-        # process deprecated params
-        logger = kwargs.get('logger', logger)
-
-        if slow_request_timeout is not None:
-            warnings.warn(
-                'slow_request_timeout is deprecated', DeprecationWarning)
+                 lingering_time=10.0):
 
         super().__init__(loop=loop)
 
-        self._loop = loop if loop is not None else asyncio.get_event_loop()
-
         self._manager = manager
-        self._time_service = manager.time_service
         self._request_handler = manager.request_handler
         self._request_factory = manager.request_factory
 
@@ -118,10 +107,9 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
         self._messages = deque()
         self._message_tail = b''
 
-        self._waiters = deque()
+        self._waiter = None
         self._error_handler = None
-        self._request_handlers = []
-        self._max_concurrent_handlers = max_concurrent_handlers
+        self._task_handler = self._loop.create_task(self.start())
 
         self._upgrade = False
         self._payload_parser = None
@@ -132,14 +120,13 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
             max_headers=max_headers,
             payload_exception=RequestPayloadError)
 
-        self.transport = None
         self._reading_paused = False
 
         self.logger = logger
         self.debug = debug
         self.access_log = access_log
         if access_log:
-            self.access_logger = helpers.AccessLogger(
+            self.access_logger = access_log_class(
                 access_log, access_log_format)
         else:
             self.access_logger = None
@@ -148,29 +135,15 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
         self._force_close = False
 
     def __repr__(self):
-        self._request = None
-        if self._request is None:
-            meth = 'none'
-            path = 'none'
-        else:
-            meth = 'none'
-            path = 'none'
-            # meth = self._request.method
-            # path = self._request.rel_url.raw_path
-        return "<{} {}:{} {}>".format(
-            self.__class__.__name__, meth, path,
+        return "<{} {}>".format(
+            self.__class__.__name__,
             'connected' if self.transport is not None else 'disconnected')
 
-    @property
-    def time_service(self):
-        return self._time_service
-
     @property
     def keepalive_timeout(self):
         return self._keepalive_timeout
 
-    @asyncio.coroutine
-    def shutdown(self, timeout=15.0):
+    async def shutdown(self, timeout=15.0):
         """Worker process is about to exit, we need cleanup everything and
         stop accepting requests. It is especially important for keep-alive
         connections."""
@@ -179,50 +152,34 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
         if self._keepalive_handle is not None:
             self._keepalive_handle.cancel()
 
-        # cancel waiters
-        for waiter in self._waiters:
-            if not waiter.done():
-                waiter.cancel()
+        if self._waiter:
+            self._waiter.cancel()
 
         # wait for handlers
         with suppress(asyncio.CancelledError, asyncio.TimeoutError):
             with CeilTimeout(timeout, loop=self._loop):
                 if self._error_handler and not self._error_handler.done():
-                    yield from self._error_handler
+                    await self._error_handler
 
-                while True:
-                    h = None
-                    for handler in self._request_handlers:
-                        if not handler.done():
-                            h = handler
-                            break
-                    if h:
-                        yield from h
-                    else:
-                        break
+                if self._task_handler and not self._task_handler.done():
+                    await self._task_handler
 
-        # force-close non-idle handlers
-        for handler in self._request_handlers:
-            if not handler.done():
-                handler.cancel()
+        # force-close non-idle handler
+        if self._task_handler:
+            self._task_handler.cancel()
 
         if self.transport is not None:
             self.transport.close()
             self.transport = None
 
-        if self._request_handlers:
-            self._request_handlers.clear()
-
     def connection_made(self, transport):
         super().connection_made(transport)
 
-        self.transport = transport
-        self.writer = StreamWriter(self, transport, self._loop)
-
         if self._tcp_keepalive:
-            tcp_keepalive(self, transport)
+            tcp_keepalive(transport)
 
-        self.writer.set_tcp_nodelay(True)
+        tcp_cork(transport, False)
+        tcp_nodelay(transport, True)
         self._manager.connection_made(self, transport)
 
     def connection_lost(self, exc):
@@ -235,24 +192,21 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
         self._request_factory = None
         self._request_handler = None
         self._request_parser = None
-        self.transport = self.writer = None
-
-        if self._payload_parser is not None:
-            self._payload_parser.feed_eof()
-            self._payload_parser = None
 
         if self._keepalive_handle is not None:
             self._keepalive_handle.cancel()
 
-        for handler in self._request_handlers:
-            if not handler.done():
-                handler.cancel()
+        if self._task_handler:
+            self._task_handler.cancel()
 
         if self._error_handler is not None:
-            if not self._error_handler.done():
-                self._error_handler.cancel()
+            self._error_handler.cancel()
 
-        self._request_handlers = ()
+        self._task_handler = None
+
+        if self._payload_parser is not None:
+            self._payload_parser.feed_eof()
+            self._payload_parser = None
 
     def set_parser(self, parser):
         assert self._payload_parser is None
@@ -276,38 +230,33 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
                 messages, upgraded, tail = self._request_parser.feed_data(data)
             except HttpProcessingError as exc:
                 # something happened during parsing
-                self.close()
-                self._error_handler = ensure_future(
+                self._error_handler = self._loop.create_task(
                     self.handle_parse_error(
-                        PayloadWriter(self.writer, self._loop),
-                        400, exc, exc.message),
-                    loop=self._loop)
+                        StreamWriter(self, self._loop),
+                        400, exc, exc.message))
+                self.close()
             except Exception as exc:
                 # 500: internal error
-                self.close()
-                self._error_handler = ensure_future(
+                self._error_handler = self._loop.create_task(
                     self.handle_parse_error(
-                        PayloadWriter(self.writer, self._loop),
-                        500, exc), loop=self._loop)
+                        StreamWriter(self, self._loop),
+                        500, exc))
+                self.close()
             else:
-                for (msg, payload) in messages:
-                    self._request_count += 1
-
-                    if self._waiters:
-                        waiter = self._waiters.popleft()
-                        waiter.set_result((msg, payload))
-                    elif self._max_concurrent_handlers:
-                        self._max_concurrent_handlers -= 1
-                        data = []
-                        handler = ensure_future(
-                            self.start(msg, payload, data), loop=self._loop)
-                        data.append(handler)
-                        self._request_handlers.append(handler)
-                    else:
+                if messages:
+                    # sometimes the parser returns no messages
+                    for (msg, payload) in messages:
+                        self._request_count += 1
                         self._messages.append((msg, payload))
 
-                self._upgraded = upgraded
-                if upgraded:
+                    waiter = self._waiter
+                    if waiter is not None:
+                        if not waiter.done():
+                            # don't set result twice
+                            waiter.set_result(None)
+
+                self._upgrade = upgraded
+                if upgraded and tail:
                     self._message_tail = tail
 
         # no parser, just store
@@ -326,29 +275,29 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
         :param bool val: new state.
         """
         self._keepalive = val
+        if self._keepalive_handle:
+            self._keepalive_handle.cancel()
+            self._keepalive_handle = None
 
     def close(self):
         """Stop accepting new pipelinig messages and close
         connection when handlers done processing messages"""
         self._close = True
-        for waiter in self._waiters:
-            if not waiter.done():
-                waiter.cancel()
+        if self._waiter:
+            self._waiter.cancel()
 
     def force_close(self):
         """Force close connection"""
         self._force_close = True
-        for waiter in self._waiters:
-            if not waiter.done():
-                waiter.cancel()
+        if self._waiter:
+            self._waiter.cancel()
         if self.transport is not None:
             self.transport.close()
             self.transport = None
 
-    def log_access(self, message, environ, response, time):
-        if self.access_logger:
-            self.access_logger.log(message, environ, response,
-                                   self.transport, time)
+    def log_access(self, request, response, time):
+        if self.access_logger is not None:
+            self.access_logger.log(request, response, time)
 
     def log_debug(self, *args, **kw):
         if self.debug:
@@ -358,20 +307,21 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
         self.logger.exception(*args, **kw)
 
     def _process_keepalive(self):
-        if self._force_close:
+        if self._force_close or not self._keepalive:
             return
 
         next = self._keepalive_time + self._keepalive_timeout
 
-        # all handlers in idle state
-        if len(self._request_handlers) == len(self._waiters):
-            now = self._time_service.loop_time
-            if now + 1.0 > next:
+        # handler in idle state
+        if self._waiter:
+            if self._loop.time() > next:
                 self.force_close()
                 return
 
-        self._keepalive_handle = self._loop.call_at(
-            next, self._process_keepalive)
+        # not all request handlers are done,
+        # reschedule itself to next second
+        self._keepalive_handle = self._loop.call_later(
+            self.KEEPALIVE_RESCHEDULE_DELAY, self._process_keepalive)
 
     def pause_reading(self):
         if not self._reading_paused:
@@ -389,9 +339,8 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
                 pass
             self._reading_paused = False
 
-    @asyncio.coroutine
-    def start(self, message, payload, handler):
-        """Start processing of incoming requests.
+    async def start(self):
+        """Process incoming request.
 
         It reads request line, request headers and request payload, then
         calls handle_request() method. Subclass has to override
@@ -400,21 +349,33 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
         keep_alive(True) specified.
         """
         loop = self._loop
-        handler = handler[0]
+        handler = self._task_handler
         manager = self._manager
         keepalive_timeout = self._keepalive_timeout
 
         while not self._force_close:
+            if not self._messages:
+                try:
+                    # wait for next request
+                    self._waiter = loop.create_future()
+                    await self._waiter
+                except asyncio.CancelledError:
+                    break
+                finally:
+                    self._waiter = None
+
+            message, payload = self._messages.popleft()
+
             if self.access_log:
                 now = loop.time()
 
             manager.requests_count += 1
-            writer = PayloadWriter(self.writer, loop)
+            writer = StreamWriter(self, loop)
             request = self._request_factory(
                 message, payload, self, writer, handler)
             try:
                 try:
-                    resp = yield from self._request_handler(request)
+                    resp = await self._request_handler(request)
                 except HTTPException as exc:
                     resp = exc
                 except asyncio.CancelledError:
@@ -425,21 +386,24 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
                     resp = self.handle_error(request, 504)
                 except Exception as exc:
                     resp = self.handle_error(request, 500, exc)
+                else:
+                    # Deprecation warning (See #2415)
+                    if isinstance(resp, HTTPException):
+                        warnings.warn(
+                            "returning HTTPException object is deprecated "
+                            "(#2415) and will be removed, "
+                            "please raise the exception instead",
+                            DeprecationWarning)
 
-                yield from resp.prepare(request)
-                yield from resp.write_eof()
+                await resp.prepare(request)
+                await resp.write_eof()
 
                 # notify server about keep-alive
                 self._keepalive = resp.keep_alive
 
-                # Restore default state.
-                # Should be no-op if server code didn't touch these attributes.
-                writer.set_tcp_cork(False)
-                writer.set_tcp_nodelay(True)
-
                 # log access
                 if self.access_log:
-                    self.log_access(message, None, resp, loop.time() - now)
+                    self.log_access(request, resp, loop.time() - now)
 
                 # check payload
                 if not payload.is_eof():
@@ -454,11 +418,11 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
 
                         with suppress(
                                 asyncio.TimeoutError, asyncio.CancelledError):
-                            while (not payload.is_eof() and now < end_t):
+                            while not payload.is_eof() and now < end_t:
                                 timeout = min(end_t - now, lingering_time)
                                 with CeilTimeout(timeout, loop=loop):
                                     # read and ignore
-                                    yield from payload.readany()
+                                    await payload.readany()
                                 now = loop.time()
 
                     # if payload still uncompleted
@@ -466,6 +430,16 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
                         self.log_debug('Uncompleted request.')
                         self.close()
 
+                payload.set_exception(PayloadAccessError())
+
+            except asyncio.CancelledError:
+                self.log_debug('Ignored premature client disconnection ')
+                break
+            except RuntimeError as exc:
+                if self.debug:
+                    self.log_exception(
+                        'Unhandled runtime exception', exc_info=exc)
+                self.force_close()
             except Exception as exc:
                 self.log_exception('Unhandled exception', exc_info=exc)
                 self.force_close()
@@ -473,36 +447,23 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
                 if self.transport is None:
                     self.log_debug('Ignored premature client disconnection.')
                 elif not self._force_close:
-                    if self._messages:
-                        message, payload = self._messages.popleft()
+                    if self._keepalive and not self._close:
+                        # start keep-alive timer
+                        if keepalive_timeout is not None:
+                            now = self._loop.time()
+                            self._keepalive_time = now
+                            if self._keepalive_handle is None:
+                                self._keepalive_handle = loop.call_at(
+                                    now + keepalive_timeout,
+                                    self._process_keepalive)
                     else:
-                        if self._keepalive and not self._close:
-                            # start keep-alive timer
-                            if keepalive_timeout is not None:
-                                now = self._time_service.loop_time
-                                self._keepalive_time = now
-                                if self._keepalive_handle is None:
-                                    self._keepalive_handle = loop.call_at(
-                                        now + keepalive_timeout,
-                                        self._process_keepalive)
-
-                            # wait for next request
-                            waiter = create_future(loop)
-                            self._waiters.append(waiter)
-                            try:
-                                message, payload = yield from waiter
-                            except asyncio.CancelledError:
-                                # shutdown process
-                                break
-                        else:
-                            break
+                        break
 
         # remove handler, close transport if no handlers left
         if not self._force_close:
-            self._request_handlers.remove(handler)
-            if not self._request_handlers:
-                if self.transport is not None:
-                    self.transport.close()
+            self._task_handler = None
+            if self.transport is not None and self._error_handler is None:
+                self.transport.close()
 
     def handle_error(self, request, status=500, exc=None, message=None):
         """Handle errors.
@@ -514,14 +475,12 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
         if status == 500:
             msg = "<h1>500 Internal Server Error</h1>"
             if self.debug:
-                try:
+                with suppress(Exception):
                     tb = traceback.format_exc()
                     tb = html_escape(tb)
                     msg += '<br><h2>Traceback:</h2>\n<pre>'
                     msg += tb
                     msg += '</pre>'
-                except:  # pragma: no cover
-                    pass
             else:
                 msg += "Server got itself in trouble"
                 msg = ("<html><head><title>500 Internal Server Error</title>"
@@ -538,17 +497,16 @@ class RequestHandler(asyncio.streams.FlowControlMixin, asyncio.Protocol):
 
         return resp
 
-    @asyncio.coroutine
-    def handle_parse_error(self, writer, status, exc=None, message=None):
+    async def handle_parse_error(self, writer, status, exc=None, message=None):
         request = BaseRequest(
             ERROR, EMPTY_PAYLOAD,
-            self, writer, self._time_service, None)
+            self, writer, None, self._loop)
 
         resp = self.handle_error(request, status, exc, message)
-        yield from resp.prepare(request)
-        yield from resp.write_eof()
+        await resp.prepare(request)
+        await resp.write_eof()
 
-        # Restore default state.
-        # Should be no-op if server code didn't touch these attributes.
-        self.writer.set_tcp_cork(False)
-        self.writer.set_tcp_nodelay(True)
+        if self.transport is not None:
+            self.transport.close()
+
+        self._error_handler = None
