@@ -1,6 +1,7 @@
 import asyncio
 import gc
 import socket
+import ssl
 import unittest
 from unittest import mock
 
@@ -8,10 +9,19 @@ from yarl import URL
 
 import aiohttp
 from aiohttp.client_reqrep import ClientRequest, ClientResponse
+from aiohttp.helpers import TimerNoop
 from aiohttp.test_utils import make_mocked_coro
 
 
 class TestProxy(unittest.TestCase):
+    response_mock_attrs = {
+        'status': 200,
+    }
+    mocked_response = mock.Mock(**response_mock_attrs)
+    clientrequest_mock_attrs = {
+        'return_value.send.return_value.start':
+            make_mocked_coro(mocked_response),
+    }
 
     def setUp(self):
         self.loop = asyncio.new_event_loop()
@@ -29,7 +39,7 @@ class TestProxy(unittest.TestCase):
         req = ClientRequest(
             'GET', URL('http://www.python.org'),
             proxy=URL('http://proxy.example.com'),
-            loop=self.loop
+            loop=self.loop,
         )
         self.assertEqual(str(req.proxy), 'http://proxy.example.com')
 
@@ -37,10 +47,13 @@ class TestProxy(unittest.TestCase):
         connector = aiohttp.TCPConnector(loop=self.loop)
         connector._resolve_host = make_mocked_coro([mock.MagicMock()])
 
-        proto = mock.Mock()
+        proto = mock.Mock(**{
+            'transport.get_extra_info.return_value': False,
+        })
         self.loop.create_connection = make_mocked_coro(
             (proto.transport, proto))
-        conn = self.loop.run_until_complete(connector.connect(req))
+        conn = self.loop.run_until_complete(
+            connector.connect(req, None, aiohttp.ClientTimeout()))
         self.assertEqual(req.url, URL('http://www.python.org'))
         self.assertIs(conn._protocol, proto)
         self.assertIs(conn.transport, proto.transport)
@@ -49,7 +62,39 @@ class TestProxy(unittest.TestCase):
             'GET', URL('http://proxy.example.com'),
             auth=None,
             headers={'Host': 'www.python.org'},
+            loop=self.loop,
+            ssl=None)
+
+    @mock.patch('aiohttp.connector.ClientRequest')
+    def test_proxy_headers(self, ClientRequestMock):
+        req = ClientRequest(
+            'GET', URL('http://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
+            proxy_headers={'Foo': 'Bar'},
             loop=self.loop)
+        self.assertEqual(str(req.proxy), 'http://proxy.example.com')
+
+        # mock all the things!
+        connector = aiohttp.TCPConnector(loop=self.loop)
+        connector._resolve_host = make_mocked_coro([mock.MagicMock()])
+
+        proto = mock.Mock(**{
+            'transport.get_extra_info.return_value': False,
+        })
+        self.loop.create_connection = make_mocked_coro(
+            (proto.transport, proto))
+        conn = self.loop.run_until_complete(connector.connect(
+            req, None, aiohttp.ClientTimeout()))
+        self.assertEqual(req.url, URL('http://www.python.org'))
+        self.assertIs(conn._protocol, proto)
+        self.assertIs(conn.transport, proto.transport)
+
+        ClientRequestMock.assert_called_with(
+            'GET', URL('http://proxy.example.com'),
+            auth=None,
+            headers={'Host': 'www.python.org', 'Foo': 'Bar'},
+            loop=self.loop,
+            ssl=None)
 
     def test_proxy_auth(self):
         with self.assertRaises(ValueError) as ctx:
@@ -63,22 +108,7 @@ class TestProxy(unittest.TestCase):
             "proxy_auth must be None or BasicAuth() tuple",
         )
 
-    @mock.patch('aiohttp.client_reqrep.PayloadWriter')
-    def _test_connect_request_with_unicode_host(self, Request_mock):
-        loop = mock.Mock()
-        request = ClientRequest("CONNECT", URL("http://éé.com/"),
-                                loop=loop)
-
-        request.response_class = mock.Mock()
-        request.write_bytes = mock.Mock()
-        request.write_bytes.return_value = asyncio.Future(loop=loop)
-        request.write_bytes.return_value.set_result(None)
-        request.send(mock.Mock())
-
-        Request_mock.assert_called_with(mock.ANY, mock.ANY, "xn--9caa.com:80",
-                                        mock.ANY, loop=loop)
-
-    def test_proxy_connection_error(self):
+    def test_proxy_dns_error(self):
         connector = aiohttp.TCPConnector(loop=self.loop)
         connector._resolve_host = make_mocked_coro(
             raise_exception=OSError('dont take it serious'))
@@ -90,114 +120,29 @@ class TestProxy(unittest.TestCase):
         )
         expected_headers = dict(req.headers)
         with self.assertRaises(aiohttp.ClientConnectorError):
-            self.loop.run_until_complete(connector.connect(req))
+            self.loop.run_until_complete(connector.connect(
+                req, None, aiohttp.ClientTimeout()))
         self.assertEqual(req.url.path, '/')
         self.assertEqual(dict(req.headers), expected_headers)
 
-    @mock.patch('aiohttp.connector.ClientRequest')
-    def test_auth(self, ClientRequestMock):
-        proxy_req = ClientRequest(
-            'GET', URL('http://proxy.example.com'),
-            auth=aiohttp.helpers.BasicAuth('user', 'pass'),
-            loop=self.loop
-        )
-        ClientRequestMock.return_value = proxy_req
-        self.assertIn('AUTHORIZATION', proxy_req.headers)
-        self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
-
+    def test_proxy_connection_error(self):
         connector = aiohttp.TCPConnector(loop=self.loop)
-        connector._resolve_host = make_mocked_coro([mock.MagicMock()])
-
-        tr, proto = mock.Mock(), mock.Mock()
-        self.loop.create_connection = make_mocked_coro((tr, proto))
+        connector._resolve_host = make_mocked_coro([{
+            'hostname': 'www.python.org',
+            'host': '127.0.0.1', 'port': 80,
+            'family': socket.AF_INET, 'proto': 0,
+            'flags': socket.AI_NUMERICHOST}])
+        connector._loop.create_connection = make_mocked_coro(
+            raise_exception=OSError('dont take it serious'))
 
         req = ClientRequest(
             'GET', URL('http://www.python.org'),
             proxy=URL('http://proxy.example.com'),
-            proxy_auth=aiohttp.helpers.BasicAuth('user', 'pass'),
             loop=self.loop,
         )
-        self.assertNotIn('AUTHORIZATION', req.headers)
-        self.assertNotIn('PROXY-AUTHORIZATION', req.headers)
-        conn = self.loop.run_until_complete(connector.connect(req))
-
-        self.assertEqual(req.url, URL('http://www.python.org'))
-        self.assertNotIn('AUTHORIZATION', req.headers)
-        self.assertIn('PROXY-AUTHORIZATION', req.headers)
-        self.assertNotIn('AUTHORIZATION', proxy_req.headers)
-        self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
-
-        ClientRequestMock.assert_called_with(
-            'GET', URL('http://proxy.example.com'),
-            auth=aiohttp.helpers.BasicAuth('user', 'pass'),
-            loop=mock.ANY, headers=mock.ANY)
-        conn.close()
-
-    def test_auth_utf8(self):
-        proxy_req = ClientRequest(
-            'GET', URL('http://proxy.example.com'),
-            auth=aiohttp.helpers.BasicAuth('юзер', 'пасс', 'utf-8'),
-            loop=self.loop)
-        self.assertIn('AUTHORIZATION', proxy_req.headers)
-
-    @mock.patch('aiohttp.connector.ClientRequest')
-    def test_auth_from_url(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET',
-                                  URL('http://user:pass@proxy.example.com'),
-                                  loop=self.loop)
-        ClientRequestMock.return_value = proxy_req
-        self.assertIn('AUTHORIZATION', proxy_req.headers)
-        self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
-
-        connector = aiohttp.TCPConnector(loop=self.loop)
-        connector._resolve_host = make_mocked_coro([mock.MagicMock()])
-
-        tr, proto = mock.Mock(), mock.Mock()
-        self.loop.create_connection = make_mocked_coro((tr, proto))
-
-        req = ClientRequest(
-            'GET', URL('http://www.python.org'),
-            proxy=URL('http://user:pass@proxy.example.com'),
-            loop=self.loop,
-        )
-        self.assertNotIn('AUTHORIZATION', req.headers)
-        self.assertNotIn('PROXY-AUTHORIZATION', req.headers)
-        conn = self.loop.run_until_complete(connector.connect(req))
-
-        self.assertEqual(req.url, URL('http://www.python.org'))
-        self.assertNotIn('AUTHORIZATION', req.headers)
-        self.assertIn('PROXY-AUTHORIZATION', req.headers)
-        self.assertNotIn('AUTHORIZATION', proxy_req.headers)
-        self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
-
-        ClientRequestMock.assert_called_with(
-            'GET', URL('http://user:pass@proxy.example.com'),
-            auth=None, loop=mock.ANY, headers=mock.ANY)
-        conn.close()
-
-    @mock.patch('aiohttp.connector.ClientRequest')
-    def test_auth__not_modifying_request(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET',
-                                  URL('http://user:pass@proxy.example.com'),
-                                  loop=self.loop)
-        ClientRequestMock.return_value = proxy_req
-        proxy_req_headers = dict(proxy_req.headers)
-
-        connector = aiohttp.TCPConnector(loop=self.loop)
-        connector._resolve_host = make_mocked_coro(
-            raise_exception=OSError('nothing personal'))
-
-        req = ClientRequest(
-            'GET', URL('http://www.python.org'),
-            proxy=URL('http://user:pass@proxy.example.com'),
-            loop=self.loop,
-        )
-        req_headers = dict(req.headers)
-        with self.assertRaises(aiohttp.ClientConnectorError):
-            self.loop.run_until_complete(connector.connect(req))
-        self.assertEqual(req.headers, req_headers)
-        self.assertEqual(req.url.path, '/')
-        self.assertEqual(proxy_req.headers, proxy_req_headers)
+        with self.assertRaises(aiohttp.ClientProxyConnectionError):
+            self.loop.run_until_complete(connector.connect(
+                req, None, aiohttp.ClientTimeout()))
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_https_connect(self, ClientRequestMock):
@@ -205,10 +150,15 @@ class TestProxy(unittest.TestCase):
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
-        proxy_resp._loop = self.loop
-        proxy_req.send = send_mock = mock.Mock()
-        send_mock.return_value = proxy_resp
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'),
+                                    request_info=mock.Mock(),
+                                    writer=mock.Mock(),
+                                    continue100=None,
+                                    timer=TimerNoop(),
+                                    traces=[],
+                                    loop=self.loop,
+                                    session=mock.Mock())
+        proxy_req.send = make_mocked_coro(proxy_resp)
         proxy_resp.start = make_mocked_coro(mock.Mock(status=200))
 
         connector = aiohttp.TCPConnector(loop=self.loop)
@@ -224,7 +174,8 @@ class TestProxy(unittest.TestCase):
             proxy=URL('http://proxy.example.com'),
             loop=self.loop,
         )
-        self.loop.run_until_complete(connector._create_connection(req))
+        self.loop.run_until_complete(
+            connector._create_connection(req, None, aiohttp.ClientTimeout()))
 
         self.assertEqual(req.url.path, '/')
         self.assertEqual(proxy_req.method, 'CONNECT')
@@ -236,16 +187,119 @@ class TestProxy(unittest.TestCase):
         proxy_resp.close()
         self.loop.run_until_complete(req.close())
 
+    @mock.patch('aiohttp.connector.ClientRequest')
+    def test_https_connect_certificate_error(self, ClientRequestMock):
+        proxy_req = ClientRequest('GET', URL('http://proxy.example.com'),
+                                  loop=self.loop)
+        ClientRequestMock.return_value = proxy_req
+
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'),
+                                    request_info=mock.Mock(),
+                                    writer=mock.Mock(),
+                                    continue100=None,
+                                    timer=TimerNoop(),
+                                    traces=[],
+                                    loop=self.loop,
+                                    session=mock.Mock())
+        proxy_req.send = make_mocked_coro(proxy_resp)
+        proxy_resp.start = make_mocked_coro(mock.Mock(status=200))
+
+        connector = aiohttp.TCPConnector(loop=self.loop)
+        connector._resolve_host = make_mocked_coro(
+            [{'hostname': 'hostname', 'host': '127.0.0.1', 'port': 80,
+              'family': socket.AF_INET, 'proto': 0, 'flags': 0}])
+
+        seq = 0
+
+        @asyncio.coroutine
+        def create_connection(*args, **kwargs):
+            nonlocal seq
+            seq += 1
+
+            # connection to http://proxy.example.com
+            if seq == 1:
+                return mock.Mock(), mock.Mock()
+            # connection to https://www.python.org
+            elif seq == 2:
+                raise ssl.CertificateError
+            else:
+                assert False
+
+        self.loop.create_connection = create_connection
+
+        req = ClientRequest(
+            'GET', URL('https://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
+            loop=self.loop,
+        )
+        with self.assertRaises(aiohttp.ClientConnectorCertificateError):
+            self.loop.run_until_complete(connector._create_connection(
+                req, None, aiohttp.ClientTimeout()))
+
+    @mock.patch('aiohttp.connector.ClientRequest')
+    def test_https_connect_ssl_error(self, ClientRequestMock):
+        proxy_req = ClientRequest('GET', URL('http://proxy.example.com'),
+                                  loop=self.loop)
+        ClientRequestMock.return_value = proxy_req
+
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'),
+                                    request_info=mock.Mock(),
+                                    writer=mock.Mock(),
+                                    continue100=None,
+                                    timer=TimerNoop(),
+                                    traces=[],
+                                    loop=self.loop,
+                                    session=mock.Mock())
+        proxy_req.send = make_mocked_coro(proxy_resp)
+        proxy_resp.start = make_mocked_coro(mock.Mock(status=200))
+
+        connector = aiohttp.TCPConnector(loop=self.loop)
+        connector._resolve_host = make_mocked_coro(
+            [{'hostname': 'hostname', 'host': '127.0.0.1', 'port': 80,
+              'family': socket.AF_INET, 'proto': 0, 'flags': 0}])
+
+        seq = 0
+
+        @asyncio.coroutine
+        def create_connection(*args, **kwargs):
+            nonlocal seq
+            seq += 1
+
+            # connection to http://proxy.example.com
+            if seq == 1:
+                return mock.Mock(), mock.Mock()
+            # connection to https://www.python.org
+            elif seq == 2:
+                raise ssl.SSLError
+            else:
+                assert False
+
+        self.loop.create_connection = create_connection
+
+        req = ClientRequest(
+            'GET', URL('https://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
+            loop=self.loop,
+        )
+        with self.assertRaises(aiohttp.ClientConnectorSSLError):
+            self.loop.run_until_complete(connector._create_connection(
+                req, None, aiohttp.ClientTimeout()))
+
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_https_connect_runtime_error(self, ClientRequestMock):
         proxy_req = ClientRequest('GET', URL('http://proxy.example.com'),
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
-        proxy_resp._loop = self.loop
-        proxy_req.send = send_mock = mock.Mock()
-        send_mock.return_value = proxy_resp
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'),
+                                    request_info=mock.Mock(),
+                                    writer=mock.Mock(),
+                                    continue100=None,
+                                    timer=TimerNoop(),
+                                    traces=[],
+                                    loop=self.loop,
+                                    session=mock.Mock())
+        proxy_req.send = make_mocked_coro(proxy_resp)
         proxy_resp.start = make_mocked_coro(mock.Mock(status=200))
 
         connector = aiohttp.TCPConnector(loop=self.loop)
@@ -264,7 +318,8 @@ class TestProxy(unittest.TestCase):
         )
         with self.assertRaisesRegex(
                 RuntimeError, "Transport does not expose socket instance"):
-            self.loop.run_until_complete(connector._create_connection(req))
+            self.loop.run_until_complete(connector._create_connection(
+                req, None, aiohttp.ClientTimeout()))
 
         self.loop.run_until_complete(proxy_req.close())
         proxy_resp.close()
@@ -276,14 +331,18 @@ class TestProxy(unittest.TestCase):
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
-        proxy_resp._loop = self.loop
-        proxy_req.send = send_mock = mock.Mock()
-        send_mock.return_value = proxy_resp
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'),
+                                    request_info=mock.Mock(),
+                                    writer=mock.Mock(),
+                                    continue100=None,
+                                    timer=TimerNoop(),
+                                    traces=[],
+                                    loop=self.loop,
+                                    session=mock.Mock())
+        proxy_req.send = make_mocked_coro(proxy_resp)
         proxy_resp.start = make_mocked_coro(
             mock.Mock(status=400, reason='bad request'))
 
-        connector = aiohttp.TCPConnector(loop=self.loop)
         connector = aiohttp.TCPConnector(loop=self.loop)
         connector._resolve_host = make_mocked_coro(
             [{'hostname': 'hostname', 'host': '127.0.0.1', 'port': 80,
@@ -300,7 +359,8 @@ class TestProxy(unittest.TestCase):
         )
         with self.assertRaisesRegex(
                 aiohttp.ClientHttpProxyError, "400, message='bad request'"):
-            self.loop.run_until_complete(connector._create_connection(req))
+            self.loop.run_until_complete(connector._create_connection(
+                req, None, aiohttp.ClientTimeout()))
 
         self.loop.run_until_complete(proxy_req.close())
         proxy_resp.close()
@@ -312,10 +372,15 @@ class TestProxy(unittest.TestCase):
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
-        proxy_resp._loop = self.loop
-        proxy_req.send = send_mock = mock.Mock()
-        send_mock.return_value = proxy_resp
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'),
+                                    request_info=mock.Mock(),
+                                    writer=mock.Mock(),
+                                    continue100=None,
+                                    timer=TimerNoop(),
+                                    traces=[],
+                                    loop=self.loop,
+                                    session=mock.Mock())
+        proxy_req.send = make_mocked_coro(proxy_resp)
         proxy_resp.start = make_mocked_coro(
             raise_exception=OSError("error message"))
 
@@ -334,7 +399,8 @@ class TestProxy(unittest.TestCase):
             loop=self.loop,
         )
         with self.assertRaisesRegex(OSError, "error message"):
-            self.loop.run_until_complete(connector._create_connection(req))
+            self.loop.run_until_complete(connector._create_connection(
+                req, None, aiohttp.ClientTimeout()))
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_request_port(self, ClientRequestMock):
@@ -356,7 +422,8 @@ class TestProxy(unittest.TestCase):
             proxy=URL('http://proxy.example.com'),
             loop=self.loop,
         )
-        self.loop.run_until_complete(connector._create_connection(req))
+        self.loop.run_until_complete(connector._create_connection(
+            req, None, aiohttp.ClientTimeout()))
         self.assertEqual(req.url, URL('http://localhost:1234/path'))
 
     def test_proxy_auth_property(self):
@@ -380,10 +447,15 @@ class TestProxy(unittest.TestCase):
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
-        proxy_resp._loop = self.loop
-        proxy_req.send = send_mock = mock.Mock()
-        send_mock.return_value = proxy_resp
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'),
+                                    request_info=mock.Mock(),
+                                    writer=mock.Mock(),
+                                    continue100=None,
+                                    timer=TimerNoop(),
+                                    traces=[],
+                                    loop=self.loop,
+                                    session=mock.Mock())
+        proxy_req.send = make_mocked_coro(proxy_resp)
         proxy_resp.start = make_mocked_coro(mock.Mock(status=200))
 
         connector = aiohttp.TCPConnector(loop=self.loop)
@@ -399,11 +471,12 @@ class TestProxy(unittest.TestCase):
             proxy=URL('http://proxy.example.com'),
             loop=self.loop,
         )
-        self.loop.run_until_complete(connector._create_connection(req))
+        self.loop.run_until_complete(connector._create_connection(
+            req, None, aiohttp.ClientTimeout()))
 
         self.loop.create_connection.assert_called_with(
             mock.ANY,
-            ssl=connector.ssl_context,
+            ssl=connector._make_ssl_context(True),
             sock=mock.ANY,
             server_hostname='www.python.org')
 
@@ -425,10 +498,15 @@ class TestProxy(unittest.TestCase):
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
-        proxy_resp._loop = self.loop
-        proxy_req.send = send_mock = mock.Mock()
-        send_mock.return_value = proxy_resp
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'),
+                                    request_info=mock.Mock(),
+                                    writer=mock.Mock(),
+                                    continue100=None,
+                                    timer=TimerNoop(),
+                                    traces=[],
+                                    loop=self.loop,
+                                    session=mock.Mock())
+        proxy_req.send = make_mocked_coro(proxy_resp)
         proxy_resp.start = make_mocked_coro(mock.Mock(status=200))
 
         connector = aiohttp.TCPConnector(loop=self.loop)
@@ -449,7 +527,8 @@ class TestProxy(unittest.TestCase):
         )
         self.assertNotIn('AUTHORIZATION', req.headers)
         self.assertNotIn('PROXY-AUTHORIZATION', req.headers)
-        self.loop.run_until_complete(connector._create_connection(req))
+        self.loop.run_until_complete(
+            connector._create_connection(req, None, aiohttp.ClientTimeout()))
 
         self.assertEqual(req.url.path, '/')
         self.assertNotIn('AUTHORIZATION', req.headers)
@@ -457,7 +536,10 @@ class TestProxy(unittest.TestCase):
         self.assertNotIn('AUTHORIZATION', proxy_req.headers)
         self.assertIn('PROXY-AUTHORIZATION', proxy_req.headers)
 
-        connector._resolve_host.assert_called_with('proxy.example.com', 80)
+        connector._resolve_host.assert_called_with(
+            'proxy.example.com',
+            80,
+            traces=mock.ANY)
 
         self.loop.run_until_complete(proxy_req.close())
         proxy_resp.close()
