@@ -25,7 +25,7 @@ def request(buf):
         return helpers.noop()
 
     async def write_headers(status_line, headers):
-        headers = status_line + ''.join(
+        headers = status_line + '\r\n' + ''.join(
             [k + ': ' + v + '\r\n' for k, v in headers.items()])
         headers = headers.encode('utf-8') + b'\r\n'
         buf.extend(headers)
