@@ -8,7 +8,7 @@ from unittest import mock
 
 import pytest
 from async_generator import async_generator, yield_
-from multidict import MultiDict
+from multidict import CIMultiDictProxy, MultiDict
 from yarl import URL
 
 import aiohttp
@@ -1822,3 +1822,31 @@ async def test_app_add_routes(aiohttp_client):
     client = await aiohttp_client(app)
     resp = await client.get('/get')
     assert resp.status == 200
+
+
+async def test_request_headers_type(aiohttp_client):
+
+    async def handler(request):
+        assert isinstance(request.headers, CIMultiDictProxy)
+        return web.Response()
+
+    app = web.Application()
+    app.add_routes([web.get('/get', handler)])
+
+    client = await aiohttp_client(app)
+    resp = await client.get('/get')
+    assert resp.status == 200
+
+
+async def test_signal_on_error_handler(aiohttp_client):
+
+    async def on_prepare(request, response):
+        response.headers['X-Custom'] = 'val'
+
+    app = web.Application()
+    app.on_response_prepare.append(on_prepare)
+
+    client = await aiohttp_client(app)
+    resp = await client.get('/')
+    assert resp.status == 404
+    assert resp.headers['X-Custom'] == 'val'
