@@ -11,7 +11,7 @@ import pytest
 from aiohttp import helpers, http, streams, web
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def make_srv(loop, manager):
     srv = None
 
@@ -38,6 +38,8 @@ def srv(make_srv, transport):
     srv = make_srv()
     srv.connection_made(transport)
     transport.close.side_effect = partial(srv.connection_lost, None)
+    srv._drain_helper = mock.Mock()
+    srv._drain_helper.side_effect = helpers.noop
     return srv
 
 
@@ -49,8 +51,7 @@ def buf():
 @pytest.fixture
 def request_handler():
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return web.Response()
 
     m = mock.Mock()
@@ -62,8 +63,7 @@ def request_handler():
 def handle_with_error():
     def wrapper(exc=ValueError):
 
-        @asyncio.coroutine
-        def handle(request):
+        async def handle(request):
             raise exc
 
         h = mock.Mock()
@@ -72,12 +72,12 @@ def handle_with_error():
     return wrapper
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def writer(srv):
-    return http.PayloadWriter(srv.writer, srv._loop)
+    return http.StreamWriter(srv, srv.transport, srv._loop)
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def transport(buf):
     transport = mock.Mock()
 
@@ -85,7 +85,7 @@ def transport(buf):
         buf.extend(chunk)
 
     transport.write.side_effect = write
-    transport.drain.side_effect = helpers.noop
+    transport.is_closing.return_value = False
 
     return transport
 
@@ -98,24 +98,17 @@ def ceil(mocker):
     mocker.patch('aiohttp.helpers.ceil').side_effect = ceil
 
 
-@asyncio.coroutine
-def test_shutdown(srv, loop, transport):
+async def test_shutdown(srv, loop, transport):
     assert transport is srv.transport
 
     srv._keepalive = True
-    srv.data_received(
-        b'GET / HTTP/1.1\r\n'
-        b'Host: example.com\r\n'
-        b'Content-Length: 0\r\n\r\n')
+    task_handler = srv._task_handler
 
-    request_handler = srv._request_handlers[-1]
-
-    yield from asyncio.sleep(0.1, loop=loop)
-    assert len(srv._waiters) == 1
-    assert len(srv._request_handlers) == 1
+    assert srv._waiter is not None
+    assert srv._task_handler is not None
 
     t0 = loop.time()
-    yield from srv.shutdown()
+    await srv.shutdown()
     t1 = loop.time()
 
     assert t1 - t0 < 0.05, t1-t0
@@ -123,87 +116,56 @@ def test_shutdown(srv, loop, transport):
     assert transport.close.called
     assert srv.transport is None
 
-    assert not srv._request_handlers
-    assert request_handler.done()
+    assert not srv._task_handler
+    await asyncio.sleep(0.1, loop=loop)
+    assert task_handler.done()
 
 
-@asyncio.coroutine
-def test_shutdown_multiple_handlers(srv, loop, transport):
-    srv.handle_request = mock.Mock()
-    srv.handle_request.side_effect = helpers.noop
-
-    assert transport is srv.transport
-
-    srv._keepalive = True
-    srv.data_received(
-        b'GET / HTTP/1.1\r\n'
-        b'Host: example.com\r\n'
-        b'Content-Length: 0\r\n\r\n'
-        b'GET / HTTP/1.1\r\n'
-        b'Host: example.com\r\n'
-        b'Content-Length: 0\r\n\r\n')
-
-    h1, h2 = srv._request_handlers
-
-    yield from asyncio.sleep(0.1, loop=loop)
-    assert len(srv._waiters) == 2
-    assert len(srv._request_handlers) == 2
-
-    t0 = loop.time()
-    yield from srv.shutdown()
-    t1 = loop.time()
-
-    assert t1 - t0 < 0.05, t1-t0
-
-    assert transport.close.called
-    assert srv.transport is None
-
-    assert not srv._request_handlers
-    assert h1.done()
-    assert h2.done()
-
-
-@asyncio.coroutine
-def test_double_shutdown(srv, transport):
-    yield from srv.shutdown()
+async def test_double_shutdown(srv, transport):
+    await srv.shutdown()
     assert transport.close.called
     assert srv.transport is None
 
     transport.reset_mock()
-    yield from srv.shutdown()
+    await srv.shutdown()
     assert not transport.close.called
     assert srv.transport is None
 
 
-@asyncio.coroutine
-def test_close_after_response(srv, loop, transport):
+async def test_shutdown_wait_error_handler(loop, srv, transport):
+
+    async def _error_handle():
+        pass
+
+    srv._error_handler = loop.create_task(_error_handle())
+    await srv.shutdown()
+    assert srv._error_handler.done()
+
+
+async def test_close_after_response(srv, loop, transport):
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
-    h, = srv._request_handlers
+    h = srv._task_handler
 
-    yield from asyncio.sleep(0.1, loop=loop)
-    assert len(srv._waiters) == 0
-    assert len(srv._request_handlers) == 0
+    await asyncio.sleep(0.1, loop=loop)
+    assert srv._waiter is None
+    assert srv._task_handler is None
 
     assert transport.close.called
     assert srv.transport is None
 
-    assert not srv._request_handlers
     assert h.done()
 
 
 def test_connection_made(make_srv):
     srv = make_srv()
-    assert not srv._request_handlers
-
     srv.connection_made(mock.Mock())
-    assert not srv._request_handlers
     assert not srv._force_close
 
 
-def test_connection_made_with_keepaplive(make_srv, transport):
+def test_connection_made_with_tcp_keepaplive(make_srv, transport):
     srv = make_srv()
 
     sock = mock.Mock()
@@ -213,7 +175,7 @@ def test_connection_made_with_keepaplive(make_srv, transport):
                                        socket.SO_KEEPALIVE, 1)
 
 
-def test_connection_made_without_keepaplive(make_srv):
+def test_connection_made_without_tcp_keepaplive(make_srv):
     srv = make_srv(tcp_keepalive=False)
 
     sock = mock.Mock()
@@ -230,23 +192,22 @@ def test_eof_received(make_srv):
     # assert srv.reader._eof
 
 
-@asyncio.coroutine
-def test_connection_lost(srv, loop):
+async def test_connection_lost(srv, loop):
     srv.data_received(
         b'GET / HTTP/1.1\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
     srv._keepalive = True
 
-    handle = srv._request_handlers[0]
-    yield from asyncio.sleep(0, loop=loop)  # wait for .start() starting
+    handle = srv._task_handler
+    await asyncio.sleep(0, loop=loop)  # wait for .start() starting
     srv.connection_lost(None)
 
     assert srv._force_close
 
-    yield from handle
+    await handle
 
-    assert not srv._request_handlers
+    assert not srv._task_handler
 
 
 def test_srv_keep_alive(srv):
@@ -259,32 +220,34 @@ def test_srv_keep_alive(srv):
     assert not srv._keepalive
 
 
-def test_slow_request(make_srv):
-    with pytest.warns(DeprecationWarning):
-        make_srv(slow_request_timeout=0.01)
+def test_srv_keep_alive_disable(srv):
+    handle = srv._keepalive_handle = mock.Mock()
+
+    srv.keep_alive(False)
+    assert not srv._keepalive
+    assert srv._keepalive_handle is None
+    handle.cancel.assert_called_with()
 
 
-@asyncio.coroutine
-def test_simple(srv, loop, buf):
+async def test_simple(srv, loop, buf):
     srv.data_received(
         b'GET / HTTP/1.1\r\n\r\n')
 
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     assert buf.startswith(b'HTTP/1.1 200 OK\r\n')
 
 
-@asyncio.coroutine
-def test_bad_method(srv, loop, buf):
+async def test_bad_method(srv, loop, buf):
     srv.data_received(
         b'!@#$ / HTTP/1.0\r\n'
         b'Host: example.com\r\n\r\n')
 
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     assert buf.startswith(b'HTTP/1.0 400 Bad Request\r\n')
 
 
-@asyncio.coroutine
-def test_internal_error(srv, loop, buf):
+async def test_data_received_error(srv, loop, buf):
+    transport = srv.transport
     srv._request_parser = mock.Mock()
     srv._request_parser.feed_data.side_effect = TypeError
 
@@ -292,31 +255,32 @@ def test_internal_error(srv, loop, buf):
         b'!@#$ / HTTP/1.0\r\n'
         b'Host: example.com\r\n\r\n')
 
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     assert buf.startswith(b'HTTP/1.0 500 Internal Server Error\r\n')
+    assert transport.close.called
+    assert srv._error_handler is None
 
 
-@asyncio.coroutine
-def test_line_too_long(srv, loop, buf):
+async def test_line_too_long(srv, loop, buf):
     srv.data_received(b''.join([b'a' for _ in range(10000)]) + b'\r\n\r\n')
 
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     assert buf.startswith(b'HTTP/1.0 400 Bad Request\r\n')
 
 
-@asyncio.coroutine
-def test_invalid_content_length(srv, loop, buf):
+async def test_invalid_content_length(srv, loop, buf):
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: sdgg\r\n\r\n')
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
 
     assert buf.startswith(b'HTTP/1.0 400 Bad Request\r\n')
 
 
-@asyncio.coroutine
-def test_handle_error__utf(make_srv, buf, transport, loop, request_handler):
+async def test_handle_error__utf(
+    make_srv, buf, transport, loop, request_handler
+):
     request_handler.side_effect = RuntimeError('что-то пошло не так')
 
     srv = make_srv(debug=True)
@@ -328,7 +292,7 @@ def test_handle_error__utf(make_srv, buf, transport, loop, request_handler):
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
 
     assert b'HTTP/1.0 500 Internal Server Error' in buf
     assert b'Content-Type: text/html; charset=utf-8' in buf
@@ -340,8 +304,34 @@ def test_handle_error__utf(make_srv, buf, transport, loop, request_handler):
         "Error handling request", exc_info=mock.ANY)
 
 
-@asyncio.coroutine
-def test_handle_uncompleted(
+async def test_unhandled_runtime_error(
+    make_srv, loop, transport, request_handler
+):
+
+    async def handle(request):
+        resp = web.Response()
+        resp.write_eof = mock.Mock()
+        resp.write_eof.side_effect = RuntimeError
+        return resp
+
+    srv = make_srv(lingering_time=0)
+    srv.debug = True
+    srv.connection_made(transport)
+    srv.logger.exception = mock.Mock()
+    request_handler.side_effect = handle
+
+    srv.data_received(
+        b'GET / HTTP/1.0\r\n'
+        b'Host: example.com\r\n'
+        b'Content-Length: 0\r\n\r\n')
+
+    await srv._task_handler
+    assert request_handler.called
+    srv.logger.exception.assert_called_with(
+        "Unhandled runtime exception", exc_info=mock.ANY)
+
+
+async def test_handle_uncompleted(
         make_srv, loop, transport, handle_with_error, request_handler):
     closed = False
 
@@ -361,15 +351,14 @@ def test_handle_uncompleted(
         b'Host: example.com\r\n'
         b'Content-Length: 50000\r\n\r\n')
 
-    yield from srv._request_handlers[0]
+    await srv._task_handler
     assert request_handler.called
     assert closed
     srv.logger.exception.assert_called_with(
         "Error handling request", exc_info=mock.ANY)
 
 
-@asyncio.coroutine
-def test_handle_uncompleted_pipe(
+async def test_handle_uncompleted_pipe(
         make_srv, loop, transport, request_handler, handle_with_error):
     closed = False
     normal_completed = False
@@ -384,11 +373,10 @@ def test_handle_uncompleted_pipe(
     srv.connection_made(transport)
     srv.logger.exception = mock.Mock()
 
-    @asyncio.coroutine
-    def handle(request):
+    async def handle(request):
         nonlocal normal_completed
         normal_completed = True
-        yield from asyncio.sleep(0.05, loop=loop)
+        await asyncio.sleep(0.05, loop=loop)
         return web.Response()
 
     # normal
@@ -397,7 +385,7 @@ def test_handle_uncompleted_pipe(
         b'GET / HTTP/1.1\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
 
     # with exception
     request_handler.side_effect = handle_with_error()
@@ -406,11 +394,11 @@ def test_handle_uncompleted_pipe(
         b'Host: example.com\r\n'
         b'Content-Length: 50000\r\n\r\n')
 
-    assert len(srv._request_handlers) == 2
+    assert srv._task_handler
 
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
 
-    yield from srv._request_handlers[0]
+    await srv._task_handler
     assert normal_completed
     assert request_handler.called
     assert closed
@@ -418,12 +406,10 @@ def test_handle_uncompleted_pipe(
         "Error handling request", exc_info=mock.ANY)
 
 
-@asyncio.coroutine
-def test_lingering(srv, loop, transport):
+async def test_lingering(srv, loop, transport):
     assert not transport.close.called
 
-    @asyncio.coroutine
-    def handle(message, request, writer):
+    async def handle(message, request, writer):
         pass
 
     srv.handle_request = handle
@@ -432,78 +418,92 @@ def test_lingering(srv, loop, transport):
         b'Host: example.com\r\n'
         b'Content-Length: 3\r\n\r\n')
 
-    yield from asyncio.sleep(0.05, loop=loop)
+    await asyncio.sleep(0.05, loop=loop)
     assert not transport.close.called
 
     srv.data_received(b'123')
 
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     transport.close.assert_called_with()
 
 
-@asyncio.coroutine
-def test_lingering_disabled(make_srv, loop, transport, request_handler):
+async def test_lingering_disabled(make_srv, loop, transport, request_handler):
 
-    @asyncio.coroutine
-    def handle_request(request):
-        yield from asyncio.sleep(0, loop=loop)
+    async def handle_request(request):
+        await asyncio.sleep(0, loop=loop)
 
     srv = make_srv(lingering_time=0)
     srv.connection_made(transport)
     request_handler.side_effect = handle_request
 
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     assert not transport.close.called
 
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 50\r\n\r\n')
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     assert not transport.close.called
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     transport.close.assert_called_with()
 
 
-@asyncio.coroutine
-def test_lingering_timeout(make_srv, loop, transport, ceil, request_handler):
+async def test_lingering_timeout(
+    make_srv, loop, transport, ceil, request_handler
+):
 
-    @asyncio.coroutine
-    def handle_request(request):
-        yield from asyncio.sleep(0, loop=loop)
+    async def handle_request(request):
+        await asyncio.sleep(0, loop=loop)
 
     srv = make_srv(lingering_time=1e-30)
     srv.connection_made(transport)
     request_handler.side_effect = handle_request
 
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     assert not transport.close.called
 
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 50\r\n\r\n')
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     assert not transport.close.called
 
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     transport.close.assert_called_with()
 
 
+async def test_handle_payload_access_error(
+    make_srv, loop, transport, request_handler
+):
+    srv = make_srv(lingering_time=0)
+    srv.connection_made(transport)
+    srv.data_received(
+        b'POST /test HTTP/1.1\r\n'
+        b'Content-Length: 9\r\n\r\n'
+        b'some data'
+    )
+    # start request_handler task
+    await asyncio.sleep(0, loop=loop)
+
+    with pytest.raises(web.PayloadAccessError):
+        await request_handler.call_args[0][0].content.read()
+
+
 def test_handle_cancel(make_srv, loop, transport):
     log = mock.Mock()
 
     srv = make_srv(logger=log, debug=True)
     srv.connection_made(transport)
 
-    def handle_request(message, payload, writer):
-        yield from asyncio.sleep(10, loop=loop)
+    async def handle_request(message, payload, writer):
+        await asyncio.sleep(10, loop=loop)
 
     srv.handle_request = handle_request
 
-    @asyncio.coroutine
-    def cancel():
-        srv._request_handlers[0].cancel()
+    async def cancel():
+        srv._task_handler.cancel()
 
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
@@ -511,7 +511,7 @@ def test_handle_cancel(make_srv, loop, transport):
         b'Host: example.com\r\n\r\n')
 
     loop.run_until_complete(
-        asyncio.gather(srv._request_handlers[0], cancel(), loop=loop))
+        asyncio.gather(srv._task_handler, cancel(), loop=loop))
     assert log.debug.called
 
 
@@ -529,15 +529,14 @@ def test_handle_cancelled(make_srv, loop, transport):
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n\r\n')
 
-    r_handler = srv._request_handlers[0]
+    r_handler = srv._task_handler
     assert loop.run_until_complete(r_handler) is None
 
 
-@asyncio.coroutine
-def test_handle_400(srv, loop, buf, transport):
+async def test_handle_400(srv, loop, buf, transport):
     srv.data_received(b'GET / HT/asd\r\n\r\n')
 
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
     assert b'400 Bad Request' in buf
 
 
@@ -547,19 +546,19 @@ def test_handle_500(srv, loop, buf, transport, request_handler):
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n\r\n')
-    loop.run_until_complete(srv._request_handlers[0])
+    loop.run_until_complete(srv._task_handler)
 
     assert b'500 Internal Server Error' in buf
 
 
-@asyncio.coroutine
-def test_keep_alive(make_srv, loop, transport, ceil):
+async def test_keep_alive(make_srv, loop, transport, ceil):
     srv = make_srv(keepalive_timeout=0.05)
+    srv.KEEPALIVE_RESCHEDULE_DELAY = 0.1
     srv.connection_made(transport)
 
     srv.keep_alive(True)
     srv.handle_request = mock.Mock()
-    srv.handle_request.return_value = helpers.create_future(loop)
+    srv.handle_request.return_value = loop.create_future()
     srv.handle_request.return_value.set_result(1)
 
     srv.data_received(
@@ -567,14 +566,15 @@ def test_keep_alive(make_srv, loop, transport, ceil):
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
 
-    yield from asyncio.sleep(0, loop=loop)
-    assert len(srv._waiters) == 1
+    await asyncio.sleep(0, loop=loop)
+    waiter = srv._waiter
+    assert waiter
     assert srv._keepalive_handle is not None
     assert not transport.close.called
 
-    yield from asyncio.sleep(0.1, loop=loop)
+    await asyncio.sleep(0.2, loop=loop)
     assert transport.close.called
-    assert srv._waiters[0].cancelled
+    assert waiter.cancelled
 
 
 def test_srv_process_request_without_timeout(make_srv, loop, transport):
@@ -585,7 +585,7 @@ def test_srv_process_request_without_timeout(make_srv, loop, transport):
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n\r\n')
 
-    loop.run_until_complete(srv._request_handlers[0])
+    loop.run_until_complete(srv._task_handler)
     assert transport.close.called
 
 
@@ -598,26 +598,24 @@ def test_keep_alive_timeout_nondefault(make_srv):
     assert 10 == srv.keepalive_timeout
 
 
-@asyncio.coroutine
-def test_supports_connect_method(srv, loop, transport, request_handler):
+async def test_supports_connect_method(srv, loop, transport, request_handler):
     srv.data_received(
         b'CONNECT aiohttp.readthedocs.org:80 HTTP/1.0\r\n'
         b'Content-Length: 0\r\n\r\n')
-    yield from asyncio.sleep(0.1, loop=loop)
+    await asyncio.sleep(0.1, loop=loop)
 
     assert request_handler.called
     assert isinstance(
         request_handler.call_args[0][0].content,
-        streams.FlowControlStreamReader)
+        streams.StreamReader)
 
 
-@asyncio.coroutine
-def test_content_length_0(srv, loop, request_handler):
+async def test_content_length_0(srv, loop, request_handler):
     srv.data_received(
         b'GET / HTTP/1.1\r\n'
         b'Host: example.org\r\n'
         b'Content-Length: 0\r\n\r\n')
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
 
     assert request_handler.called
     assert request_handler.call_args[0][0].content == streams.EMPTY_PAYLOAD
@@ -646,10 +644,8 @@ def test_rudimentary_transport(srv, loop):
     assert not srv._reading_paused
 
 
-@asyncio.coroutine
-def test_close(srv, loop, transport):
+async def test_close(srv, loop, transport):
     transport.close.side_effect = partial(srv.connection_lost, None)
-    srv._max_concurrent_handlers = 2
     srv.connection_made(transport)
 
     srv.handle_request = mock.Mock()
@@ -666,26 +662,25 @@ def test_close(srv, loop, transport):
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
 
-    yield from asyncio.sleep(0, loop=loop)
-    assert len(srv._request_handlers) == 2
-    assert len(srv._waiters) == 2
+    await asyncio.sleep(0, loop=loop)
+    assert srv._task_handler
+    assert srv._waiter
 
     srv.close()
-    yield from asyncio.sleep(0, loop=loop)
-    assert len(srv._request_handlers) == 0
+    await asyncio.sleep(0, loop=loop)
+    assert srv._task_handler is None
     assert srv.transport is None
     assert transport.close.called
 
 
-@asyncio.coroutine
-def test_pipeline_multiple_messages(srv, loop, transport, request_handler):
+async def test_pipeline_multiple_messages(
+    srv, loop, transport, request_handler
+):
     transport.close.side_effect = partial(srv.connection_lost, None)
-    srv._max_concurrent_handlers = 1
 
     processed = 0
 
-    @asyncio.coroutine
-    def handle(request):
+    async def handle(request):
         nonlocal processed
         processed += 1
         return web.Response()
@@ -703,31 +698,31 @@ def test_pipeline_multiple_messages(srv, loop, transport, request_handler):
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
 
-    assert len(srv._request_handlers) == 1
-    assert len(srv._messages) == 1
-    assert len(srv._waiters) == 0
+    assert srv._task_handler is not None
+    assert len(srv._messages) == 2
+    assert srv._waiter is not None
 
-    yield from asyncio.sleep(0, loop=loop)
-    assert len(srv._request_handlers) == 1
-    assert len(srv._waiters) == 1
+    await asyncio.sleep(0, loop=loop)
+    assert srv._task_handler is not None
+    assert srv._waiter is not None
     assert processed == 2
 
 
-@asyncio.coroutine
-def test_pipeline_response_order(srv, loop, buf, transport, request_handler):
+async def test_pipeline_response_order(
+    srv, loop, buf, transport, request_handler
+):
     transport.close.side_effect = partial(srv.connection_lost, None)
     srv._keepalive = True
 
     processed = []
 
-    @asyncio.coroutine
-    def handle1(request):
+    async def handle1(request):
         nonlocal processed
-        yield from asyncio.sleep(0.01, loop=loop)
+        await asyncio.sleep(0.01, loop=loop)
         resp = web.StreamResponse()
-        yield from resp.prepare(request)
-        yield from resp.write(b'test1')
-        yield from resp.write_eof()
+        await resp.prepare(request)
+        await resp.write(b'test1')
+        await resp.write_eof()
         processed.append(1)
         return resp
 
@@ -736,16 +731,16 @@ def test_pipeline_response_order(srv, loop, buf, transport, request_handler):
         b'GET / HTTP/1.1\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
 
     # second
-    @asyncio.coroutine
-    def handle2(request):
+
+    async def handle2(request):
         nonlocal processed
         resp = web.StreamResponse()
-        yield from resp.prepare(request)
-        resp.write(b'test2')
-        yield from resp.write_eof()
+        await resp.prepare(request)
+        await resp.write(b'test2')
+        await resp.write_eof()
         processed.append(2)
         return resp
 
@@ -754,9 +749,88 @@ def test_pipeline_response_order(srv, loop, buf, transport, request_handler):
         b'GET / HTTP/1.1\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
-    yield from asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0, loop=loop)
 
-    assert len(srv._request_handlers) == 2
+    assert srv._task_handler is not None
 
-    yield from asyncio.sleep(0.1, loop=loop)
+    await asyncio.sleep(0.1, loop=loop)
     assert processed == [1, 2]
+
+
+def test_data_received_close(srv):
+    srv.close()
+    srv.data_received(
+        b'GET / HTTP/1.1\r\n'
+        b'Host: example.com\r\n'
+        b'Content-Length: 0\r\n\r\n')
+
+    assert not srv._messages
+
+
+def test_data_received_force_close(srv):
+    srv.force_close()
+    srv.data_received(
+        b'GET / HTTP/1.1\r\n'
+        b'Host: example.com\r\n'
+        b'Content-Length: 0\r\n\r\n')
+
+    assert not srv._messages
+
+
+async def test__process_keepalive(loop, srv):
+    # wait till the waiter is waiting
+    await asyncio.sleep(0)
+
+    assert srv._waiter is not None
+
+    srv._keepalive_time = 1
+    srv._keepalive = True
+    srv._keepalive_timeout = 1
+    expired_time = srv._keepalive_time + srv._keepalive_timeout + 1
+    with mock.patch.object(loop, "time", return_value=expired_time):
+        srv._process_keepalive()
+        assert srv._force_close
+
+
+async def test__process_keepalive_schedule_next(loop, srv):
+    # wait till the waiter is waiting
+    await asyncio.sleep(0)
+
+    srv._keepalive = True
+    srv._keepalive_time = 1
+    srv._keepalive_timeout = 1
+    expire_time = srv._keepalive_time + srv._keepalive_timeout
+    with mock.patch.object(loop, "time", return_value=expire_time):
+        with mock.patch.object(loop, "call_later") as call_later_patched:
+            srv._process_keepalive()
+            call_later_patched.assert_called_with(
+                1,
+                srv._process_keepalive
+            )
+
+
+def test__process_keepalive_force_close(loop, srv):
+    srv._force_close = True
+    with mock.patch.object(loop, "call_at") as call_at_patched:
+        srv._process_keepalive()
+        assert not call_at_patched.called
+
+
+def test_two_data_received_without_waking_up_start_task(srv, loop):
+    # make a chance to srv.start() method start waiting for srv._waiter
+    loop.run_until_complete(asyncio.sleep(0.01))
+    assert srv._waiter is not None
+
+    srv.data_received(
+        b'GET / HTTP/1.1\r\n'
+        b'Host: ex.com\r\n'
+        b'Content-Length: 1\r\n\r\n'
+        b'a')
+    srv.data_received(
+        b'GET / HTTP/1.1\r\n'
+        b'Host: ex.com\r\n'
+        b'Content-Length: 1\r\n\r\n'
+        b'b')
+
+    assert len(srv._messages) == 2
+    assert srv._waiter.done()
