@@ -1,6 +1,7 @@
 import asyncio
 import os
 import pathlib
+import zlib
 
 import pytest
 
@@ -10,7 +11,7 @@ from aiohttp import web
 
 try:
     import ssl
-except:
+except ImportError:
     ssl = False
 
 
@@ -24,96 +25,87 @@ def sender(request):
     return maker
 
 
-@asyncio.coroutine
-def test_static_file_ok(loop, test_client, sender):
+async def test_static_file_ok(aiohttp_client, sender):
     filepath = pathlib.Path(__file__).parent / 'data.unknown_mime_type'
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/')
+    resp = await client.get('/')
     assert resp.status == 200
-    txt = yield from resp.text()
+    txt = await resp.text()
     assert 'file content' == txt.rstrip()
     assert 'application/octet-stream' == resp.headers['Content-Type']
     assert resp.headers.get('Content-Encoding') is None
-    yield from resp.release()
+    await resp.release()
 
 
-@asyncio.coroutine
-def test_static_file_ok_string_path(loop, test_client, sender):
+async def test_static_file_ok_string_path(aiohttp_client, sender):
     filepath = pathlib.Path(__file__).parent / 'data.unknown_mime_type'
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(str(filepath))
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/')
+    resp = await client.get('/')
     assert resp.status == 200
-    txt = yield from resp.text()
+    txt = await resp.text()
     assert 'file content' == txt.rstrip()
     assert 'application/octet-stream' == resp.headers['Content-Type']
     assert resp.headers.get('Content-Encoding') is None
-    yield from resp.release()
+    await resp.release()
 
 
-@asyncio.coroutine
-def test_static_file_not_exists(loop, test_client):
+async def test_static_file_not_exists(aiohttp_client):
 
     app = web.Application()
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/fake')
+    resp = await client.get('/fake')
     assert resp.status == 404
-    yield from resp.release()
+    await resp.release()
 
 
-@asyncio.coroutine
-def test_static_file_name_too_long(loop, test_client):
+async def test_static_file_name_too_long(aiohttp_client):
 
     app = web.Application()
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/x*500')
+    resp = await client.get('/x*500')
     assert resp.status == 404
-    yield from resp.release()
+    await resp.release()
 
 
-@asyncio.coroutine
-def test_static_file_upper_directory(loop, test_client):
+async def test_static_file_upper_directory(aiohttp_client):
 
     app = web.Application()
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/../../')
+    resp = await client.get('/../../')
     assert resp.status == 404
-    yield from resp.release()
+    await resp.release()
 
 
-@asyncio.coroutine
-def test_static_file_with_content_type(loop, test_client, sender):
+async def test_static_file_with_content_type(aiohttp_client, sender):
     filepath = (pathlib.Path(__file__).parent / 'aiohttp.jpg')
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath, chunk_size=16)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/')
+    resp = await client.get('/')
     assert resp.status == 200
-    body = yield from resp.read()
+    body = await resp.read()
     with filepath.open('rb') as f:
         content = f.read()
         assert content == body
@@ -122,23 +114,21 @@ def test_static_file_with_content_type(loop, test_client, sender):
     resp.close()
 
 
-@asyncio.coroutine
-def test_static_file_custom_content_type(loop, test_client, sender):
+async def test_static_file_custom_content_type(aiohttp_client, sender):
     filepath = (pathlib.Path(__file__).parent / 'hello.txt.gz')
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         resp = sender(filepath, chunk_size=16)
         resp.content_type = 'application/pdf'
         return resp
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/')
+    resp = await client.get('/')
     assert resp.status == 200
-    body = yield from resp.read()
+    body = await resp.read()
     with filepath.open('rb') as f:
         content = f.read()
         assert content == body
@@ -147,44 +137,41 @@ def test_static_file_custom_content_type(loop, test_client, sender):
     resp.close()
 
 
-@asyncio.coroutine
-def test_static_file_custom_content_type_compress(loop, test_client, sender):
+async def test_static_file_custom_content_type_compress(aiohttp_client,
+                                                        sender):
     filepath = (pathlib.Path(__file__).parent / 'hello.txt')
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         resp = sender(filepath, chunk_size=16)
         resp.content_type = 'application/pdf'
         return resp
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/')
+    resp = await client.get('/')
     assert resp.status == 200
-    body = yield from resp.read()
+    body = await resp.read()
     assert b'hello aiohttp\n' == body
     assert resp.headers['Content-Type'] == 'application/pdf'
     assert resp.headers.get('Content-Encoding') == 'gzip'
     resp.close()
 
 
-@asyncio.coroutine
-def test_static_file_with_content_encoding(loop, test_client, sender):
+async def test_static_file_with_content_encoding(aiohttp_client, sender):
     filepath = pathlib.Path(__file__).parent / 'hello.txt.gz'
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/')
+    resp = await client.get('/')
     assert 200 == resp.status
-    body = yield from resp.read()
+    body = await resp.read()
     assert b'hello aiohttp\n' == body
     ct = resp.headers['CONTENT-TYPE']
     assert 'text/plain' == ct
@@ -193,90 +180,84 @@ def test_static_file_with_content_encoding(loop, test_client, sender):
     resp.close()
 
 
-@asyncio.coroutine
-def test_static_file_if_modified_since(loop, test_client, sender):
+async def test_static_file_if_modified_since(aiohttp_client, sender):
     filename = 'data.unknown_mime_type'
     filepath = pathlib.Path(__file__).parent / filename
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/')
+    resp = await client.get('/')
     assert 200 == resp.status
     lastmod = resp.headers.get('Last-Modified')
     assert lastmod is not None
     resp.close()
 
-    resp = yield from client.get('/', headers={'If-Modified-Since': lastmod})
-    body = yield from resp.read()
+    resp = await client.get('/', headers={'If-Modified-Since': lastmod})
+    body = await resp.read()
     assert 304 == resp.status
     assert resp.headers.get('Content-Length') is None
     assert b'' == body
     resp.close()
 
 
-@asyncio.coroutine
-def test_static_file_if_modified_since_past_date(loop, test_client, sender):
+async def test_static_file_if_modified_since_past_date(aiohttp_client, sender):
     filename = 'data.unknown_mime_type'
     filepath = pathlib.Path(__file__).parent / filename
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
     lastmod = 'Mon, 1 Jan 1990 01:01:01 GMT'
 
-    resp = yield from client.get('/', headers={'If-Modified-Since': lastmod})
+    resp = await client.get('/', headers={'If-Modified-Since': lastmod})
     assert 200 == resp.status
     resp.close()
 
 
-@asyncio.coroutine
-def test_static_file_if_modified_since_invalid_date(loop, test_client, sender):
+async def test_static_file_if_modified_since_invalid_date(aiohttp_client,
+                                                          sender):
     filename = 'data.unknown_mime_type'
     filepath = pathlib.Path(__file__).parent / filename
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
     lastmod = 'not a valid HTTP-date'
 
-    resp = yield from client.get('/', headers={'If-Modified-Since': lastmod})
+    resp = await client.get('/', headers={'If-Modified-Since': lastmod})
     assert 200 == resp.status
     resp.close()
 
 
-@asyncio.coroutine
-def test_static_file_if_modified_since_future_date(loop, test_client, sender):
+async def test_static_file_if_modified_since_future_date(aiohttp_client,
+                                                         sender):
     filename = 'data.unknown_mime_type'
     filepath = pathlib.Path(__file__).parent / filename
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(app)
 
     lastmod = 'Fri, 31 Dec 9999 23:59:59 GMT'
 
-    resp = yield from client.get('/', headers={'If-Modified-Since': lastmod})
-    body = yield from resp.read()
+    resp = await client.get('/', headers={'If-Modified-Since': lastmod})
+    body = await resp.read()
     assert 304 == resp.status
     assert resp.headers.get('Content-Length') is None
     assert b'' == body
@@ -284,8 +265,7 @@ def test_static_file_if_modified_since_future_date(loop, test_client, sender):
 
 
 @pytest.mark.skipif(not ssl, reason="ssl not supported")
-@asyncio.coroutine
-def test_static_file_ssl(loop, test_server, test_client):
+async def test_static_file_ssl(aiohttp_server, aiohttp_client):
     dirname = os.path.dirname(__file__)
     filename = 'data.unknown_mime_type'
     ssl_ctx = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
@@ -295,40 +275,39 @@ def test_static_file_ssl(loop, test_server, test_client):
     )
     app = web.Application()
     app.router.add_static('/static', dirname)
-    server = yield from test_server(app, ssl=ssl_ctx)
-    conn = aiohttp.TCPConnector(verify_ssl=False, loop=loop)
-    client = yield from test_client(server, connector=conn)
+    server = await aiohttp_server(app, ssl=ssl_ctx)
+    conn = aiohttp.TCPConnector(ssl=False)
+    client = await aiohttp_client(server, connector=conn)
 
-    resp = yield from client.get('/static/'+filename)
+    resp = await client.get('/static/'+filename)
     assert 200 == resp.status
-    txt = yield from resp.text()
+    txt = await resp.text()
     assert 'file content' == txt.rstrip()
     ct = resp.headers['CONTENT-TYPE']
     assert 'application/octet-stream' == ct
     assert resp.headers.get('CONTENT-ENCODING') is None
 
 
-@asyncio.coroutine
-def test_static_file_directory_traversal_attack(loop, test_client):
+async def test_static_file_directory_traversal_attack(loop, aiohttp_client):
     dirname = os.path.dirname(__file__)
     relpath = '../README.rst'
     assert os.path.isfile(os.path.join(dirname, relpath))
 
     app = web.Application()
     app.router.add_static('/static', dirname)
-    client = yield from test_client(app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/static/'+relpath)
+    resp = await client.get('/static/'+relpath)
     assert 404 == resp.status
 
     url_relpath2 = '/static/dir/../' + relpath
-    resp = yield from client.get(url_relpath2)
+    resp = await client.get(url_relpath2)
     assert 404 == resp.status
 
     url_abspath = \
         '/static/' + os.path.abspath(os.path.join(dirname, relpath))
-    resp = yield from client.get(url_abspath)
-    assert 404 == resp.status
+    resp = await client.get(url_abspath)
+    assert 403 == resp.status
 
 
 def test_static_route_path_existence_check():
@@ -340,8 +319,7 @@ def test_static_route_path_existence_check():
         web.StaticResource("/", nodirectory)
 
 
-@asyncio.coroutine
-def test_static_file_huge(loop, test_client, tmpdir):
+async def test_static_file_huge(loop, aiohttp_client, tmpdir):
     filename = 'huge_data.unknown_mime_type'
 
     # fill 100MB file
@@ -353,9 +331,9 @@ def test_static_file_huge(loop, test_client, tmpdir):
 
     app = web.Application()
     app.router.add_static('/static', str(tmpdir))
-    client = yield from test_client(app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/static/'+filename)
+    resp = await client.get('/static/'+filename)
     assert 200 == resp.status
     ct = resp.headers['CONTENT-TYPE']
     assert 'application/octet-stream' == ct
@@ -366,7 +344,7 @@ def test_static_file_huge(loop, test_client, tmpdir):
     off = 0
     cnt = 0
     while off < file_st.st_size:
-        chunk = yield from resp.content.readany()
+        chunk = await resp.content.readany()
         expected = f.read(len(chunk))
         assert chunk == expected
         off += len(chunk)
@@ -374,23 +352,23 @@ def test_static_file_huge(loop, test_client, tmpdir):
     f.close()
 
 
-@asyncio.coroutine
-def test_static_file_range(loop, test_client, sender):
+async def test_static_file_range(loop, aiohttp_client, sender):
     filepath = (pathlib.Path(__file__).parent.parent / 'LICENSE.txt')
 
-    @asyncio.coroutine
-    def handler(request):
+    filesize = filepath.stat().st_size
+
+    async def handler(request):
         return sender(filepath, chunk_size=16)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(lambda loop: app)
 
     with filepath.open('rb') as f:
         content = f.read()
 
     # Ensure the whole file requested in parts is correct
-    responses = yield from asyncio.gather(
+    responses = await asyncio.gather(
         client.get('/', headers={'Range': 'bytes=0-999'}),
         client.get('/', headers={'Range': 'bytes=1000-1999'}),
         client.get('/', headers={'Range': 'bytes=2000-'}),
@@ -399,12 +377,19 @@ def test_static_file_range(loop, test_client, sender):
     assert len(responses) == 3
     assert responses[0].status == 206, \
         "failed 'bytes=0-999': %s" % responses[0].reason
+    assert responses[0].headers['Content-Range'] == 'bytes 0-999/{0}'.format(
+        filesize), 'failed: Content-Range Error'
     assert responses[1].status == 206, \
         "failed 'bytes=1000-1999': %s" % responses[1].reason
+    assert responses[1].headers['Content-Range'] == \
+        'bytes 1000-1999/{0}'.format(filesize), 'failed: Content-Range Error'
     assert responses[2].status == 206, \
         "failed 'bytes=2000-': %s" % responses[2].reason
+    assert responses[2].headers['Content-Range'] == \
+        'bytes 2000-{0}/{1}'.format(filesize - 1, filesize), \
+        'failed: Content-Range Error'
 
-    body = yield from asyncio.gather(
+    body = await asyncio.gather(
         *(resp.read() for resp in responses),
         loop=loop
     )
@@ -420,117 +405,350 @@ def test_static_file_range(loop, test_client, sender):
     assert content == b"".join(body)
 
 
-@asyncio.coroutine
-def test_static_file_range_end_bigger_than_size(loop, test_client, sender):
+async def test_static_file_range_end_bigger_than_size(
+    loop,
+    aiohttp_client,
+    sender
+):
     filepath = (pathlib.Path(__file__).parent / 'aiohttp.png')
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath, chunk_size=16)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(lambda loop: app)
 
     with filepath.open('rb') as f:
         content = f.read()
 
         # Ensure the whole file requested in parts is correct
-        response = yield from client.get(
+        response = await client.get(
             '/', headers={'Range': 'bytes=61000-62000'})
 
         assert response.status == 206, \
             "failed 'bytes=61000-62000': %s" % response.reason
+        assert response.headers['Content-Range'] == \
+            'bytes 61000-61107/61108', 'failed: Content-Range Error'
 
-        body = yield from response.read()
+        body = await response.read()
         assert len(body) == 108, \
-            "failed 'bytes=0-999', received %d bytes" % len(body[0])
+            "failed 'bytes=61000-62000', received %d bytes" % len(body)
 
         assert content[61000:] == body
 
 
-@asyncio.coroutine
-def test_static_file_range_beyond_eof(loop, test_client, sender):
+async def test_static_file_range_beyond_eof(loop, aiohttp_client, sender):
     filepath = (pathlib.Path(__file__).parent / 'aiohttp.png')
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath, chunk_size=16)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(lambda loop: app)
 
     # Ensure the whole file requested in parts is correct
-    response = yield from client.get(
+    response = await client.get(
         '/', headers={'Range': 'bytes=1000000-1200000'})
 
-    assert response.status == 206, \
+    assert response.status == 416, \
         "failed 'bytes=1000000-1200000': %s" % response.reason
-    assert response.headers['content-length'] == '0'
 
 
-@asyncio.coroutine
-def test_static_file_range_tail(loop, test_client, sender):
+async def test_static_file_range_tail(loop, aiohttp_client, sender):
     filepath = (pathlib.Path(__file__).parent / 'aiohttp.png')
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath, chunk_size=16)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(lambda loop: app)
 
     with filepath.open('rb') as f:
         content = f.read()
 
     # Ensure the tail of the file is correct
-    resp = yield from client.get('/', headers={'Range': 'bytes=-500'})
+    resp = await client.get('/', headers={'Range': 'bytes=-500'})
     assert resp.status == 206, resp.reason
-    body4 = yield from resp.read()
+    assert resp.headers['Content-Range'] == 'bytes 60608-61107/61108', \
+        'failed: Content-Range Error'
+    body4 = await resp.read()
     resp.close()
     assert content[-500:] == body4
 
+    # Ensure out-of-range tails could be handled
+    resp2 = await client.get('/', headers={'Range': 'bytes=-99999999999999'})
+    assert resp2.status == 206, resp.reason
+    assert resp2.headers['Content-Range'] == 'bytes 0-61107/61108', \
+        'failed: Content-Range Error'
 
-@asyncio.coroutine
-def test_static_file_invalid_range(loop, test_client, sender):
+
+async def test_static_file_invalid_range(loop, aiohttp_client, sender):
     filepath = (pathlib.Path(__file__).parent / 'aiohttp.png')
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return sender(filepath, chunk_size=16)
 
     app = web.Application()
     app.router.add_get('/', handler)
-    client = yield from test_client(lambda loop: app)
+    client = await aiohttp_client(lambda loop: app)
 
     # range must be in bytes
-    resp = yield from client.get('/', headers={'Range': 'blocks=0-10'})
+    resp = await client.get('/', headers={'Range': 'blocks=0-10'})
     assert resp.status == 416, 'Range must be in bytes'
     resp.close()
 
     # start > end
-    resp = yield from client.get('/', headers={'Range': 'bytes=100-0'})
+    resp = await client.get('/', headers={'Range': 'bytes=100-0'})
     assert resp.status == 416, "Range start can't be greater than end"
     resp.close()
 
     # start > end
-    resp = yield from client.get('/', headers={'Range': 'bytes=10-9'})
+    resp = await client.get('/', headers={'Range': 'bytes=10-9'})
     assert resp.status == 416, "Range start can't be greater than end"
     resp.close()
 
     # non-number range
-    resp = yield from client.get('/', headers={'Range': 'bytes=a-f'})
+    resp = await client.get('/', headers={'Range': 'bytes=a-f'})
     assert resp.status == 416, 'Range must be integers'
     resp.close()
 
     # double dash range
-    resp = yield from client.get('/', headers={'Range': 'bytes=0--10'})
+    resp = await client.get('/', headers={'Range': 'bytes=0--10'})
     assert resp.status == 416, 'double dash in range'
     resp.close()
 
     # no range
-    resp = yield from client.get('/', headers={'Range': 'bytes=-'})
+    resp = await client.get('/', headers={'Range': 'bytes=-'})
     assert resp.status == 416, 'no range given'
     resp.close()
+
+
+async def test_static_file_if_unmodified_since_past_with_range(
+        aiohttp_client, sender):
+    filename = 'data.unknown_mime_type'
+    filepath = pathlib.Path(__file__).parent / filename
+
+    async def handler(request):
+        return sender(filepath)
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app)
+
+    lastmod = 'Mon, 1 Jan 1990 01:01:01 GMT'
+
+    resp = await client.get('/', headers={
+        'If-Unmodified-Since': lastmod,
+        'Range': 'bytes=2-'})
+    assert 412 == resp.status
+    resp.close()
+
+
+async def test_static_file_if_unmodified_since_future_with_range(
+        aiohttp_client, sender):
+    filename = 'data.unknown_mime_type'
+    filepath = pathlib.Path(__file__).parent / filename
+
+    async def handler(request):
+        return sender(filepath)
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app)
+
+    lastmod = 'Fri, 31 Dec 9999 23:59:59 GMT'
+
+    resp = await client.get('/', headers={
+        'If-Unmodified-Since': lastmod,
+        'Range': 'bytes=2-'})
+    assert 206 == resp.status
+    assert resp.headers['Content-Range'] == 'bytes 2-12/13'
+    assert resp.headers['Content-Length'] == '11'
+    resp.close()
+
+
+async def test_static_file_if_range_past_with_range(
+        aiohttp_client, sender):
+    filename = 'data.unknown_mime_type'
+    filepath = pathlib.Path(__file__).parent / filename
+
+    async def handler(request):
+        return sender(filepath)
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app)
+
+    lastmod = 'Mon, 1 Jan 1990 01:01:01 GMT'
+
+    resp = await client.get('/', headers={
+        'If-Range': lastmod,
+        'Range': 'bytes=2-'})
+    assert 200 == resp.status
+    assert resp.headers['Content-Length'] == '13'
+    resp.close()
+
+
+async def test_static_file_if_range_future_with_range(
+        aiohttp_client, sender):
+    filename = 'data.unknown_mime_type'
+    filepath = pathlib.Path(__file__).parent / filename
+
+    async def handler(request):
+        return sender(filepath)
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app)
+
+    lastmod = 'Fri, 31 Dec 9999 23:59:59 GMT'
+
+    resp = await client.get('/', headers={
+        'If-Range': lastmod,
+        'Range': 'bytes=2-'})
+    assert 206 == resp.status
+    assert resp.headers['Content-Range'] == 'bytes 2-12/13'
+    assert resp.headers['Content-Length'] == '11'
+    resp.close()
+
+
+async def test_static_file_if_unmodified_since_past_without_range(
+        aiohttp_client, sender):
+    filename = 'data.unknown_mime_type'
+    filepath = pathlib.Path(__file__).parent / filename
+
+    async def handler(request):
+        return sender(filepath)
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app)
+
+    lastmod = 'Mon, 1 Jan 1990 01:01:01 GMT'
+
+    resp = await client.get('/', headers={'If-Unmodified-Since': lastmod})
+    assert 412 == resp.status
+    resp.close()
+
+
+async def test_static_file_if_unmodified_since_future_without_range(
+        aiohttp_client, sender):
+    filename = 'data.unknown_mime_type'
+    filepath = pathlib.Path(__file__).parent / filename
+
+    async def handler(request):
+        return sender(filepath)
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app)
+
+    lastmod = 'Fri, 31 Dec 9999 23:59:59 GMT'
+
+    resp = await client.get('/', headers={'If-Unmodified-Since': lastmod})
+    assert 200 == resp.status
+    assert resp.headers['Content-Length'] == '13'
+    resp.close()
+
+
+async def test_static_file_if_range_past_without_range(
+        aiohttp_client, sender):
+    filename = 'data.unknown_mime_type'
+    filepath = pathlib.Path(__file__).parent / filename
+
+    async def handler(request):
+        return sender(filepath)
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app)
+
+    lastmod = 'Mon, 1 Jan 1990 01:01:01 GMT'
+
+    resp = await client.get('/', headers={'If-Range': lastmod})
+    assert 200 == resp.status
+    assert resp.headers['Content-Length'] == '13'
+    resp.close()
+
+
+async def test_static_file_if_range_future_without_range(
+        aiohttp_client, sender):
+    filename = 'data.unknown_mime_type'
+    filepath = pathlib.Path(__file__).parent / filename
+
+    async def handler(request):
+        return sender(filepath)
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app)
+
+    lastmod = 'Fri, 31 Dec 9999 23:59:59 GMT'
+
+    resp = await client.get('/', headers={'If-Range': lastmod})
+    assert 200 == resp.status
+    assert resp.headers['Content-Length'] == '13'
+    resp.close()
+
+
+async def test_static_file_if_unmodified_since_invalid_date(aiohttp_client,
+                                                            sender):
+    filename = 'data.unknown_mime_type'
+    filepath = pathlib.Path(__file__).parent / filename
+
+    async def handler(request):
+        return sender(filepath)
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app)
+
+    lastmod = 'not a valid HTTP-date'
+
+    resp = await client.get('/', headers={'If-Unmodified-Since': lastmod})
+    assert 200 == resp.status
+    resp.close()
+
+
+async def test_static_file_if_range_invalid_date(aiohttp_client,
+                                                 sender):
+    filename = 'data.unknown_mime_type'
+    filepath = pathlib.Path(__file__).parent / filename
+
+    async def handler(request):
+        return sender(filepath)
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app)
+
+    lastmod = 'not a valid HTTP-date'
+
+    resp = await client.get('/', headers={'If-Range': lastmod})
+    assert 200 == resp.status
+    resp.close()
+
+
+async def test_static_file_compression(aiohttp_client, sender):
+    filepath = pathlib.Path(__file__).parent / 'data.unknown_mime_type'
+
+    async def handler(request):
+        ret = sender(filepath)
+        ret.enable_compression()
+        return ret
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    client = await aiohttp_client(app, auto_decompress=False)
+
+    resp = await client.get('/')
+    assert resp.status == 200
+    zcomp = zlib.compressobj(wbits=-zlib.MAX_WBITS)
+    expected_body = zcomp.compress(b'file content\n') + zcomp.flush()
+    assert expected_body == await resp.read()
+    assert 'application/octet-stream' == resp.headers['Content-Type']
+    assert resp.headers.get('Content-Encoding') == 'deflate'
+    await resp.release()
