@@ -1,5 +1,6 @@
 import random
 import struct
+import zlib
 from unittest import mock
 
 import pytest
@@ -7,28 +8,43 @@ import pytest
 import aiohttp
 from aiohttp import http_websocket
 from aiohttp.http import WebSocketError, WSCloseCode, WSMessage, WSMsgType
-from aiohttp.http_websocket import (PACK_CLOSE_CODE, PACK_LEN1, PACK_LEN2,
-                                    PACK_LEN3, WebSocketReader,
-                                    _websocket_mask)
+from aiohttp.http_websocket import (_WS_DEFLATE_TRAILING, PACK_CLOSE_CODE,
+                                    PACK_LEN1, PACK_LEN2, PACK_LEN3,
+                                    WebSocketReader, _websocket_mask)
 
 
-def build_frame(message, opcode, use_mask=False, noheader=False):
+def build_frame(message, opcode, use_mask=False, noheader=False, is_fin=True,
+                compress=False):
     """Send a frame over the websocket with message as its payload."""
+    if compress:
+        compressobj = zlib.compressobj(wbits=-9)
+        message = compressobj.compress(message)
+        message = message + compressobj.flush(zlib.Z_SYNC_FLUSH)
+        if message.endswith(_WS_DEFLATE_TRAILING):
+            message = message[:-4]
     msg_length = len(message)
     if use_mask:  # pragma: no cover
         mask_bit = 0x80
     else:
         mask_bit = 0
 
+    if is_fin:
+        header_first_byte = 0x80 | opcode
+    else:
+        header_first_byte = opcode
+
+    if compress:
+        header_first_byte |= 0x40
+
     if msg_length < 126:
         header = PACK_LEN1(
-            0x80 | opcode, msg_length | mask_bit)
+            header_first_byte, msg_length | mask_bit)
     elif msg_length < (1 << 16):  # pragma: no cover
         header = PACK_LEN2(
-            0x80 | opcode, 126 | mask_bit, msg_length)
+            header_first_byte, 126 | mask_bit, msg_length)
     else:
         header = PACK_LEN3(
-            0x80 | opcode, 127 | mask_bit, msg_length)
+            header_first_byte, 127 | mask_bit, msg_length)
 
     if use_mask:  # pragma: no cover
         mask = random.randrange(0, 0xffffffff)
@@ -62,47 +78,47 @@ def out(loop):
 
 @pytest.fixture()
 def parser(out):
-    return WebSocketReader(out)
+    return WebSocketReader(out, 4*1024*1024)
 
 
 def test_parse_frame(parser):
     parser.parse_frame(struct.pack('!BB', 0b00000001, 0b00000001))
     res = parser.parse_frame(b'1')
-    fin, opcode, payload = res[0]
+    fin, opcode, payload, compress = res[0]
 
-    assert (0, 1, b'1') == (fin, opcode, payload)
+    assert (0, 1, b'1', False) == (fin, opcode, payload, not not compress)
 
 
 def test_parse_frame_length0(parser):
-    fin, opcode, payload = parser.parse_frame(
+    fin, opcode, payload, compress = parser.parse_frame(
         struct.pack('!BB', 0b00000001, 0b00000000))[0]
 
-    assert (0, 1, b'') == (fin, opcode, payload)
+    assert (0, 1, b'', False) == (fin, opcode, payload, not not compress)
 
 
 def test_parse_frame_length2(parser):
     parser.parse_frame(struct.pack('!BB', 0b00000001, 126))
     parser.parse_frame(struct.pack('!H', 4))
     res = parser.parse_frame(b'1234')
-    fin, opcode, payload = res[0]
+    fin, opcode, payload, compress = res[0]
 
-    assert (0, 1, b'1234') == (fin, opcode, payload)
+    assert (0, 1, b'1234', False) == (fin, opcode, payload, not not compress)
 
 
 def test_parse_frame_length4(parser):
     parser.parse_frame(struct.pack('!BB', 0b00000001, 127))
     parser.parse_frame(struct.pack('!Q', 4))
-    fin, opcode, payload = parser.parse_frame(b'1234')[0]
+    fin, opcode, payload, compress = parser.parse_frame(b'1234')[0]
 
-    assert (0, 1, b'1234') == (fin, opcode, payload)
+    assert (0, 1, b'1234', False) == (fin, opcode, payload, not not compress)
 
 
 def test_parse_frame_mask(parser):
     parser.parse_frame(struct.pack('!BB', 0b00000001, 0b10000001))
     parser.parse_frame(b'0001')
-    fin, opcode, payload = parser.parse_frame(b'1')[0]
+    fin, opcode, payload, compress = parser.parse_frame(b'1')[0]
 
-    assert (0, 1, b'\x01') == (fin, opcode, payload)
+    assert (0, 1, b'\x01', False) == (fin, opcode, payload, not not compress)
 
 
 def test_parse_frame_header_reversed_bits(out, parser):
@@ -117,13 +133,6 @@ def test_parse_frame_header_control_frame(out, parser):
         raise out.exception()
 
 
-def test_parse_frame_header_continuation(out, parser):
-    with pytest.raises(WebSocketError):
-        parser._frame_fin = True
-        parser.parse_frame(struct.pack('!BB', 0b00000000, 0b00000000))
-        raise out.exception()
-
-
 def _test_parse_frame_header_new_data_err(out, parser):
     with pytest.raises(WebSocketError):
         parser.parse_frame(struct.pack('!BB', 0b000000000, 0b00000000))
@@ -138,7 +147,7 @@ def test_parse_frame_header_payload_size(out, parser):
 
 def test_ping_frame(out, parser):
     parser.parse_frame = mock.Mock()
-    parser.parse_frame.return_value = [(1, WSMsgType.PING, b'data')]
+    parser.parse_frame.return_value = [(1, WSMsgType.PING, b'data', False)]
 
     parser.feed_data(b'')
     res = out._buffer[0]
@@ -147,7 +156,7 @@ def test_ping_frame(out, parser):
 
 def test_pong_frame(out, parser):
     parser.parse_frame = mock.Mock()
-    parser.parse_frame.return_value = [(1, WSMsgType.PONG, b'data')]
+    parser.parse_frame.return_value = [(1, WSMsgType.PONG, b'data', False)]
 
     parser.feed_data(b'')
     res = out._buffer[0]
@@ -156,7 +165,7 @@ def test_pong_frame(out, parser):
 
 def test_close_frame(out, parser):
     parser.parse_frame = mock.Mock()
-    parser.parse_frame.return_value = [(1, WSMsgType.CLOSE, b'')]
+    parser.parse_frame.return_value = [(1, WSMsgType.CLOSE, b'', False)]
 
     parser.feed_data(b'')
     res = out._buffer[0]
@@ -165,7 +174,7 @@ def test_close_frame(out, parser):
 
 def test_close_frame_info(out, parser):
     parser.parse_frame = mock.Mock()
-    parser.parse_frame.return_value = [(1, WSMsgType.CLOSE, b'0112345')]
+    parser.parse_frame.return_value = [(1, WSMsgType.CLOSE, b'0112345', False)]
 
     parser.feed_data(b'')
     res = out._buffer[0]
@@ -174,7 +183,7 @@ def test_close_frame_info(out, parser):
 
 def test_close_frame_invalid(out, parser):
     parser.parse_frame = mock.Mock()
-    parser.parse_frame.return_value = [(1, WSMsgType.CLOSE, b'1')]
+    parser.parse_frame.return_value = [(1, WSMsgType.CLOSE, b'1', False)]
     parser.feed_data(b'')
 
     assert isinstance(out.exception(), WebSocketError)
@@ -202,7 +211,7 @@ def test_close_frame_unicode_err(parser):
 
 def test_unknown_frame(out, parser):
     parser.parse_frame = mock.Mock()
-    parser.parse_frame.return_value = [(1, WSMsgType.CONTINUATION, b'')]
+    parser.parse_frame.return_value = [(1, WSMsgType.CONTINUATION, b'', False)]
 
     with pytest.raises(WebSocketError):
         parser.feed_data(b'')
@@ -227,20 +236,28 @@ def test_simple_text_unicode_err(parser):
 
 def test_simple_binary(out, parser):
     parser.parse_frame = mock.Mock()
-    parser.parse_frame.return_value = [(1, WSMsgType.BINARY, b'binary')]
+    parser.parse_frame.return_value = [(1, WSMsgType.BINARY, b'binary', False)]
 
     parser.feed_data(b'')
     res = out._buffer[0]
     assert res == ((WSMsgType.BINARY, b'binary', ''), 6)
 
 
-def test_continuation(out, parser):
-    parser.parse_frame = mock.Mock()
-    parser.parse_frame.return_value = [
-        (0, WSMsgType.TEXT, b'line1'),
-        (1, WSMsgType.CONTINUATION, b'line2')]
+def test_fragmentation_header(out, parser):
+    data = build_frame(b'a', WSMsgType.TEXT)
+    parser._feed_data(data[:1])
+    parser._feed_data(data[1:])
 
-    parser._feed_data(b'')
+    res = out._buffer[0]
+    assert res == (WSMessage(WSMsgType.TEXT, 'a', ''), 1)
+
+
+def test_continuation(out, parser):
+    data1 = build_frame(b'line1', WSMsgType.TEXT, is_fin=False)
+    parser._feed_data(data1)
+
+    data2 = build_frame(b'line2', WSMsgType.CONTINUATION)
+    parser._feed_data(data2)
 
     res = out._buffer[0]
     assert res == (WSMessage(WSMsgType.TEXT, 'line1line2', ''), 10)
@@ -249,12 +266,20 @@ def test_continuation(out, parser):
 def test_continuation_with_ping(out, parser):
     parser.parse_frame = mock.Mock()
     parser.parse_frame.return_value = [
-        (0, WSMsgType.TEXT, b'line1'),
-        (0, WSMsgType.PING, b''),
-        (1, WSMsgType.CONTINUATION, b'line2'),
+        (0, WSMsgType.TEXT, b'line1', False),
+        (0, WSMsgType.PING, b'', False),
+        (1, WSMsgType.CONTINUATION, b'line2', False),
     ]
 
-    parser.feed_data(b'')
+    data1 = build_frame(b'line1', WSMsgType.TEXT, is_fin=False)
+    parser._feed_data(data1)
+
+    data2 = build_frame(b'', WSMsgType.PING)
+    parser._feed_data(data2)
+
+    data3 = build_frame(b'line2', WSMsgType.CONTINUATION)
+    parser._feed_data(data3)
+
     res = out._buffer[0]
     assert res == (WSMessage(WSMsgType.PING, b'', ''), 0)
     res = out._buffer[1]
@@ -264,8 +289,8 @@ def test_continuation_with_ping(out, parser):
 def test_continuation_err(out, parser):
     parser.parse_frame = mock.Mock()
     parser.parse_frame.return_value = [
-        (0, WSMsgType.TEXT, b'line1'),
-        (1, WSMsgType.TEXT, b'line2')]
+        (0, WSMsgType.TEXT, b'line1', False),
+        (1, WSMsgType.TEXT, b'line2', False)]
 
     with pytest.raises(WebSocketError):
         parser._feed_data(b'')
@@ -274,10 +299,10 @@ def test_continuation_err(out, parser):
 def test_continuation_with_close(out, parser):
     parser.parse_frame = mock.Mock()
     parser.parse_frame.return_value = [
-        (0, WSMsgType.TEXT, b'line1'),
+        (0, WSMsgType.TEXT, b'line1', False),
         (0, WSMsgType.CLOSE,
-         build_close_frame(1002, b'test', noheader=True)),
-        (1, WSMsgType.CONTINUATION, b'line2'),
+         build_close_frame(1002, b'test', noheader=True), False),
+        (1, WSMsgType.CONTINUATION, b'line2', False),
     ]
 
     parser.feed_data(b'')
@@ -290,10 +315,10 @@ def test_continuation_with_close(out, parser):
 def test_continuation_with_close_unicode_err(out, parser):
     parser.parse_frame = mock.Mock()
     parser.parse_frame.return_value = [
-        (0, WSMsgType.TEXT, b'line1'),
+        (0, WSMsgType.TEXT, b'line1', False),
         (0, WSMsgType.CLOSE,
-         build_close_frame(1000, b'\xf4\x90\x80\x80', noheader=True)),
-        (1, WSMsgType.CONTINUATION, b'line2')]
+         build_close_frame(1000, b'\xf4\x90\x80\x80', noheader=True), False),
+        (1, WSMsgType.CONTINUATION, b'line2', False)]
 
     with pytest.raises(WebSocketError) as ctx:
         parser._feed_data(b'')
@@ -304,10 +329,10 @@ def test_continuation_with_close_unicode_err(out, parser):
 def test_continuation_with_close_bad_code(out, parser):
     parser.parse_frame = mock.Mock()
     parser.parse_frame.return_value = [
-        (0, WSMsgType.TEXT, b'line1'),
+        (0, WSMsgType.TEXT, b'line1', False),
         (0, WSMsgType.CLOSE,
-         build_close_frame(1, b'test', noheader=True)),
-        (1, WSMsgType.CONTINUATION, b'line2')]
+         build_close_frame(1, b'test', noheader=True), False),
+        (1, WSMsgType.CONTINUATION, b'line2', False)]
 
     with pytest.raises(WebSocketError) as ctx:
         parser._feed_data(b'')
@@ -318,9 +343,9 @@ def test_continuation_with_close_bad_code(out, parser):
 def test_continuation_with_close_bad_payload(out, parser):
     parser.parse_frame = mock.Mock()
     parser.parse_frame.return_value = [
-        (0, WSMsgType.TEXT, b'line1'),
-        (0, WSMsgType.CLOSE, b'1'),
-        (1, WSMsgType.CONTINUATION, b'line2')]
+        (0, WSMsgType.TEXT, b'line1', False),
+        (0, WSMsgType.CLOSE, b'1', False),
+        (1, WSMsgType.CONTINUATION, b'line2', False)]
 
     with pytest.raises(WebSocketError) as ctx:
         parser._feed_data(b'')
@@ -331,9 +356,9 @@ def test_continuation_with_close_bad_payload(out, parser):
 def test_continuation_with_close_empty(out, parser):
     parser.parse_frame = mock.Mock()
     parser.parse_frame.return_value = [
-        (0, WSMsgType.TEXT, b'line1'),
-        (0, WSMsgType.CLOSE, b''),
-        (1, WSMsgType.CONTINUATION, b'line2'),
+        (0, WSMsgType.TEXT, b'line1', False),
+        (0, WSMsgType.CLOSE, b'', False),
+        (1, WSMsgType.CONTINUATION, b'line2', False),
     ]
 
     parser.feed_data(b'')
@@ -389,3 +414,76 @@ def test_msgtype_aliases():
     assert aiohttp.WSMsgType.CLOSE == aiohttp.WSMsgType.close
     assert aiohttp.WSMsgType.CLOSED == aiohttp.WSMsgType.closed
     assert aiohttp.WSMsgType.ERROR == aiohttp.WSMsgType.error
+
+
+def test_parse_compress_frame_single(parser):
+    parser.parse_frame(struct.pack('!BB', 0b11000001, 0b00000001))
+    res = parser.parse_frame(b'1')
+    fin, opcode, payload, compress = res[0]
+
+    assert (1, 1, b'1', True) == (fin, opcode, payload, not not compress)
+
+
+def test_parse_compress_frame_multi(parser):
+    parser.parse_frame(struct.pack('!BB', 0b01000001, 126))
+    parser.parse_frame(struct.pack('!H', 4))
+    res = parser.parse_frame(b'1234')
+    fin, opcode, payload, compress = res[0]
+    assert (0, 1, b'1234', True) == (fin, opcode, payload, not not compress)
+
+    parser.parse_frame(struct.pack('!BB', 0b10000001, 126))
+    parser.parse_frame(struct.pack('!H', 4))
+    res = parser.parse_frame(b'1234')
+    fin, opcode, payload, compress = res[0]
+    assert (1, 1, b'1234', True) == (fin, opcode, payload, not not compress)
+
+    parser.parse_frame(struct.pack('!BB', 0b10000001, 126))
+    parser.parse_frame(struct.pack('!H', 4))
+    res = parser.parse_frame(b'1234')
+    fin, opcode, payload, compress = res[0]
+    assert (1, 1, b'1234', False) == (fin, opcode, payload, not not compress)
+
+
+def test_parse_compress_error_frame(parser):
+    parser.parse_frame(struct.pack('!BB', 0b01000001, 0b00000001))
+    parser.parse_frame(b'1')
+
+    with pytest.raises(WebSocketError) as ctx:
+        parser.parse_frame(struct.pack('!BB', 0b11000001, 0b00000001))
+        parser.parse_frame(b'1')
+
+    assert ctx.value.code == WSCloseCode.PROTOCOL_ERROR
+
+
+def test_parse_no_compress_frame_single():
+    parser_no_compress = WebSocketReader(out, 0, compress=False)
+    with pytest.raises(WebSocketError) as ctx:
+        parser_no_compress.parse_frame(struct.pack(
+            '!BB', 0b11000001, 0b00000001))
+        parser_no_compress.parse_frame(b'1')
+
+    assert ctx.value.code == WSCloseCode.PROTOCOL_ERROR
+
+
+def test_msg_too_large(out):
+    parser = WebSocketReader(out, 256, compress=False)
+    data = build_frame(b'text'*256, WSMsgType.TEXT)
+    with pytest.raises(WebSocketError) as ctx:
+        parser._feed_data(data)
+    assert ctx.value.code == WSCloseCode.MESSAGE_TOO_BIG
+
+
+def test_msg_too_large_not_fin(out):
+    parser = WebSocketReader(out, 256, compress=False)
+    data = build_frame(b'text'*256, WSMsgType.TEXT, is_fin=False)
+    with pytest.raises(WebSocketError) as ctx:
+        parser._feed_data(data)
+    assert ctx.value.code == WSCloseCode.MESSAGE_TOO_BIG
+
+
+def test_compressed_msg_too_large(out):
+    parser = WebSocketReader(out, 256, compress=True)
+    data = build_frame(b'aaa'*256, WSMsgType.TEXT, compress=True)
+    with pytest.raises(WebSocketError) as ctx:
+        parser._feed_data(data)
+    assert ctx.value.code == WSCloseCode.MESSAGE_TOO_BIG
